class Cursor(object,IDisposable,ISerializable):
 """
 Represents the image used to paint the mouse pointer.

 

 Cursor(handle: IntPtr)

 Cursor(stream: Stream)

 Cursor(fileName: str)

 Cursor(type: Type,resource: str)
 """
 def CopyHandle(self):
  """
  CopyHandle(self: Cursor) -> IntPtr

  

   Copies the handle of this System.Windows.Forms.Cursor.

   Returns: An System.IntPtr that represents the cursor's handle.
  """
  pass
 def Dispose(self):
  """
  Dispose(self: Cursor)

   Releases all resources used by the System.Windows.Forms.Cursor.
  """
  pass
 def Draw(self,g,targetRect):
  """
  Draw(self: Cursor,g: Graphics,targetRect: Rectangle)

   Draws the cursor on the specified surface,within the specified bounds.

  

   g: The System.Drawing.Graphics surface on which to draw the System.Windows.Forms.Cursor.

   targetRect: The System.Drawing.Rectangle that represents the bounds of the System.Windows.Forms.Cursor.
  """
  pass
 def DrawStretched(self,g,targetRect):
  """
  DrawStretched(self: Cursor,g: Graphics,targetRect: Rectangle)

   Draws the cursor in a stretched format on the specified surface,within the specified bounds.

  

   g: The System.Drawing.Graphics surface on which to draw the System.Windows.Forms.Cursor.

   targetRect: The System.Drawing.Rectangle that represents the bounds of the System.Windows.Forms.Cursor.
  """
  pass
 def Equals(self,obj):
  """
  Equals(self: Cursor,obj: object) -> bool

  

   Returns a value indicating whether this cursor is equal to the specified 

    System.Windows.Forms.Cursor.

  

  

   obj: The System.Windows.Forms.Cursor to compare.

   Returns: true if this cursor is equal to the specified System.Windows.Forms.Cursor; otherwise,false.
  """
  pass
 def GetHashCode(self):
  """
  GetHashCode(self: Cursor) -> int

  

   Retrieves the hash code for the current System.Windows.Forms.Cursor.

   Returns: A hash code for the current System.Windows.Forms.Cursor.
  """
  pass
 @staticmethod
 def Hide():
  """
  Hide()

   Hides the cursor.
  """
  pass
 @staticmethod
 def Show():
  """
  Show()

   Displays the cursor.
  """
  pass
 def ToString(self):
  """
  ToString(self: Cursor) -> str

  

   Retrieves a human readable string representing this System.Windows.Forms.Cursor.

   Returns: A System.String that represents this System.Windows.Forms.Cursor.
  """
  pass
 def __enter__(self,*args):
  """
  __enter__(self: IDisposable) -> object

  

   Provides the implementation of __enter__ for objects which implement IDisposable.
  """
  pass
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==y """
  pass
 def __exit__(self,*args):
  """
  __exit__(self: IDisposable,exc_type: object,exc_value: object,exc_back: object)

   Provides the implementation of __exit__ for objects which implement IDisposable.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,*__args):
  """
  __new__(cls: type,handle: IntPtr)

  __new__(cls: type,fileName: str)

  __new__(cls: type,type: Type,resource: str)

  __new__(cls: type,stream: Stream)
  """
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __repr__(self,*args):
  """ __repr__(self: object) -> str """
  pass
 def __str__(self,*args):
  pass
 Handle=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the handle of the cursor.



Get: Handle(self: Cursor) -> IntPtr



"""

 HotSpot=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the cursor hot spot.



Get: HotSpot(self: Cursor) -> Point



"""

 Size=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the size of the cursor object.



Get: Size(self: Cursor) -> Size



"""

 Tag=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the object that contains data about the System.Windows.Forms.Cursor.



Get: Tag(self: Cursor) -> object



Set: Tag(self: Cursor)=value

"""


 Clip=None
 Current=None
 Position=None




class ArraySegment(object,IList[T],ICollection[T],IEnumerable[T],IEnumerable,IReadOnlyList[T],IReadOnlyCollection[T]):
 """
 ArraySegment[T](array: Array[T])

 ArraySegment[T](array: Array[T],offset: int,count: int)
 """
 def Equals(self,obj):
  """
  Equals(self: ArraySegment[T],obj: ArraySegment[T]) -> bool

  

   Determines whether the specified System.ArraySegment structure is equal to the current instance.

  

   obj: The System.ArraySegment structure to be compared with the current instance.

   Returns: true if the specified System.ArraySegment structure is equal to the current instance; otherwise,

    false.

  

  Equals(self: ArraySegment[T],obj: object) -> bool

  

   Determines whether the specified object is equal to the current instance.

  

   obj: The object to be compared with the current instance.

   Returns: true if the specified object is a System.ArraySegment structure and is equal to the current 

    instance; otherwise,false.
  """
  pass
 def GetHashCode(self):
  """
  GetHashCode(self: ArraySegment[T]) -> int

  

   Returns the hash code for the current instance.

   Returns: A 32-bit signed integer hash code.
  """
  pass
 def __contains__(self,*args):
  """
  __contains__(self: ICollection[T],item: T) -> bool

  

   Determines whether the System.Collections.Generic.ICollection contains a specific value.

  

   item: The object to locate in the System.Collections.Generic.ICollection.

   Returns: true if item is found in the System.Collections.Generic.ICollection; otherwise,false.
  """
  pass
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==y """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __iter__(self,*args):
  """ __iter__(self: IEnumerable) -> object """
  pass
 def __len__(self,*args):
  """ x.__len__() <==> len(x) """
  pass
 @staticmethod
 def __new__(self,array,offset=None,count=None):
  """
  __new__(cls: type,array: Array[T])

  __new__(cls: type,array: Array[T],offset: int,count: int)

  __new__[ArraySegment`1]() -> ArraySegment[T]
  """
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __repr__(self,*args):
  """ __repr__(self: object) -> str """
  pass
 def __str__(self,*args):
  pass
 Array=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the original array containing the range of elements that the array segment delimits.



Get: Array(self: ArraySegment[T]) -> Array[T]



"""

 Count=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the number of elements in the range delimited by the array segment.



Get: Count(self: ArraySegment[T]) -> int



"""

 Offset=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the position of the first element in the range delimited by the array segment,relative to the start of the original array.



Get: Offset(self: ArraySegment[T]) -> int



"""





class ItemsChangedEventArgs(EventArgs):
 """ Provides data for the System.Windows.Controls.ItemContainerGenerator.ItemsChanged event. """
 Action=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the action that occurred on the items collection.



Get: Action(self: ItemsChangedEventArgs) -> NotifyCollectionChangedAction



"""

 ItemCount=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the number of items that were involved in the change.



Get: ItemCount(self: ItemsChangedEventArgs) -> int



"""

 ItemUICount=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the number of user interface (UI) elements involved in the change.



Get: ItemUICount(self: ItemsChangedEventArgs) -> int



"""

 OldPosition=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the position in the collection before the change occurred.



Get: OldPosition(self: ItemsChangedEventArgs) -> GeneratorPosition



"""

 Position=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the position in the collection where the change occurred.



Get: Position(self: ItemsChangedEventArgs) -> GeneratorPosition



"""





class InkCanvasStrokeErasingEventArgs(CancelEventArgs):
 """ Provides data for the System.Windows.Controls.InkCanvas.StrokeErasing event. """
 Stroke=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the stroke that is about to be erased.



Get: Stroke(self: InkCanvasStrokeErasingEventArgs) -> Stroke



"""





class AmbientProperties(object):
 """
 Provides ambient property values to top-level controls.

 

 AmbientProperties()
 """
 BackColor=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the ambient background color of an object.



Get: BackColor(self: AmbientProperties) -> Color



Set: BackColor(self: AmbientProperties)=value

"""

 Cursor=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the ambient cursor of an object.



Get: Cursor(self: AmbientProperties) -> Cursor



Set: Cursor(self: AmbientProperties)=value

"""

 Font=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the ambient font of an object.



Get: Font(self: AmbientProperties) -> Font



Set: Font(self: AmbientProperties)=value

"""

 ForeColor=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the ambient foreground color of an object.



Get: ForeColor(self: AmbientProperties) -> Color



Set: ForeColor(self: AmbientProperties)=value

"""





from typing import Tuple, Set, Iterable, List


class DisplayingOptionsDialogEventArgs(RevitAPIPreEventArgs):
    def AddTab(self, newTabName: str, tabbedDialogExtension: TabbedDialogExtension) -> None: ...
    @property
    def PagesCount(self) -> int: ...


class ApplicationClosingEventArgs(RevitAPIPreEventArgs):


class DialogBoxShowingEventArgs(RevitAPIPreEventArgs):
    @property
    def DialogId(self) -> str: ...
    def OverrideResult(self, resultCode: int) -> bool: ...


class MessageBoxShowingEventArgs(DialogBoxShowingEventArgs):
    @property
    def Message(self) -> str: ...
    @property
    def DialogType(self) -> int: ...


class TaskDialogShowingEventArgs(DialogBoxShowingEventArgs):
    @property
    def Message(self) -> str: ...


class DockableFrameFocusChangedEventArgs(RevitAPISingleEventArgs):
    @property
    def FocusGained(self) -> bool: ...
    @property
    def PaneId(self) -> DockablePaneId: ...


class DockableFrameVisibilityChangedEventArgs(RevitAPISingleEventArgs):
    @property
    def DockableFrameShown(self) -> bool: ...
    @property
    def PaneId(self) -> DockablePaneId: ...


class FabricationPartBrowserChangedEventArgs(RevitAPISingleEventArgs):
    @property
    def Operation(self) -> FabricationPartBrowserOperation: ...
    @property
    def ServiceId(self) -> int: ...
    @property
    def NumberOfSolutions(self) -> int: ...
    def GetFabricationPartTypeIds(self) -> ISet: ...
    def GetRequiredFabricationPartTypeIds(self) -> ISet: ...
    def GetFilteredSolutionsPartsTypeCounts(self) -> IDictionary: ...
    def GetAllSolutionsPartsTypeCounts(self) -> IDictionary: ...
    def GetCurrentSolutionPartTypeIds(self) -> ISet: ...


class FormulaEditingEventArgs(RevitAPIPreEventArgs):
    @property
    def CurrentDocument(self) -> Document: ...
    @property
    def ParameterId(self) -> ElementId: ...
    @property
    def Formula(self) -> str: ...
    def Apply(self, formula: str) -> None: ...


class IdlingEventArgs(RevitAPIPreEventArgs):
    def SetRaiseWithoutDelay(self) -> None: ...


class TransferringProjectStandardsEventArgs(RevitAPIPreDocEventArgs):
    @property
    def SourceDocument(self) -> Document: ...
    @property
    def TargetDocument(self) -> Document: ...
    def GetExternalItems(self) -> IDictionary: ...
    def SetExternalItems(self, externalItems: IDictionary) -> None: ...


class TransferredProjectStandardsEventArgs(RevitAPISingleEventArgs):
    @property
    def SourceDocument(self) -> Document: ...
    @property
    def TargetDocument(self) -> Document: ...
    def GetSelectedExternalItems(self) -> IDictionary: ...


class ViewActivatedEventArgs(RevitAPIPostDocEventArgs):
    @property
    def CurrentActiveView(self) -> View: ...
    @property
    def PreviousActiveView(self) -> View: ...


class ViewActivatingEventArgs(RevitAPIPreDocEventArgs):
    @property
    def NewActiveView(self) -> View: ...
    @property
    def CurrentActiveView(self) -> View: ...


class RibbonItemEventArgs:
    def __init__(self): ...
    @property
    def Application(self) -> UIApplication: ...


class ComboBoxDropDownOpenedEventArgs(RibbonItemEventArgs):
    def __init__(self): ...


class ComboBoxDropDownClosedEventArgs(RibbonItemEventArgs):
    def __init__(self): ...


class ComboBoxCurrentChangedEventArgs(RibbonItemEventArgs):
    @property
    def OldValue(self) -> ComboBoxMember: ...
    @property
    def NewValue(self) -> ComboBoxMember: ...


class TextBoxEnterPressedEventArgs(RibbonItemEventArgs):
    def __init__(self): ...


class CommandEventArgs(RevitEventArgs):
    @property
    def ActiveDocument(self) -> Document: ...
    @property
    def CommandId(self) -> RevitCommandId: ...


class BeforeExecutedEventArgs(CommandEventArgs):
    @property
    def UsingCommandData(self) -> bool: ...
    @UsingCommandData.setter
    def UsingCommandData(self, usingCommandData: bool) -> None: ...


class ExecutedEventArgs(CommandEventArgs):
    def GetJournalData(self) -> IDictionary: ...
    def SetJournalData(self, data: IDictionary) -> None: ...


class CanExecuteEventArgs(CommandEventArgs):
    @property
    def CanExecute(self) -> bool: ...
    @CanExecute.setter
    def CanExecute(self, canExecute: bool) -> None: ...
    def GetSelectedCategoryIds(self) -> ICollection: ...
    @property
    def IsChecked(self) -> bool: ...
    @IsChecked.setter
    def IsChecked(self, isChecked: bool) -> None: ...


class FabricationPartBrowserOperation:
    ShowBrowser = 0
    HideBrowser = 1
    ShowService = 2
    StartRoutingSolutionMode = 3
    CreatedRoutingSolutions = 4
    RoutingSolutionChanged = 5
    FinishRoutingSolutionMode = 6
    StartMPRoutingMode = 7
    FinishMPRoutingMode = 8
    StartServiceSwapOutMode = 9
    FinishServiceSwapOutMode = 10
    UpdateServiceSwapOutPartTypes = 11
    UpdateServiceSwapOutSizes = 12
    StartSizeSwapOutMode = 13
    FinishSizeSwapOutMode = 14
    SwitchToInlineFittingsTab = 15
    UnknownOperation = -1


class DialogBoxData(APIObject):
    @property
    def HelpId(self) -> int: ...
    def OverrideResult(self, result: int) -> bool: ...


class MessageBoxData(DialogBoxData):
    @property
    def Message(self) -> str: ...
    @property
    def DialogType(self) -> int: ...



class FlowLayoutSettings(LayoutSettings):
 """ Collects the characteristics associated with flow layouts. """
 def GetFlowBreak(self,child):
  """
  GetFlowBreak(self: FlowLayoutSettings,child: object) -> bool

  

   Returns a value that represents the flow break setting of the control.

  

   child: The child control.

   Returns: true if the flow break is set; otherwise,false.
  """
  pass
 def SetFlowBreak(self,child,value):
  """
  SetFlowBreak(self: FlowLayoutSettings,child: object,value: bool)

   Sets the value that represents the flow break setting of the control.

  

   child: The child control.

   value: The flow break value to set.
  """
  pass
 FlowDirection=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value indicating the flow direction of consecutive controls.



Get: FlowDirection(self: FlowLayoutSettings) -> FlowDirection



Set: FlowDirection(self: FlowLayoutSettings)=value

"""

 LayoutEngine=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the current flow layout engine.



Get: LayoutEngine(self: FlowLayoutSettings) -> LayoutEngine



"""

 WrapContents=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value indicating whether the contents should be wrapped or clipped when they exceed the original boundaries of their container.



Get: WrapContents(self: FlowLayoutSettings) -> bool



Set: WrapContents(self: FlowLayoutSettings)=value

"""





class TableLayoutControlCollection(ControlCollection,IList,ICollection,IEnumerable,ICloneable):
 """
 Represents a collection of child controls in a table layout container.

 

 TableLayoutControlCollection(container: TableLayoutPanel)
 """
 def Add(self,*__args):
  """
  Add(self: TableLayoutControlCollection,control: Control,column: int,row: int)

   Adds the specified control to the collection and positions it at the specified cell.

  

   control: The control to add.

   column: The column in which control will be placed.

   row: The row in which control will be placed.
  """
  pass
 def __add__(self,*args):
  """ x.__add__(y) <==> x+y """
  pass
 def __getitem__(self,*args):
  """ x.__getitem__(y) <==> x[y]x.__getitem__(y) <==> x[y] """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __iter__(self,*args):
  """ __iter__(self: IEnumerable) -> object """
  pass
 @staticmethod
 def __new__(self,container):
  """ __new__(cls: type,container: TableLayoutPanel) """
  pass
 Container=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the parent System.Windows.Forms.TableLayoutPanel that contains the controls in the collection.



Get: Container(self: TableLayoutControlCollection) -> TableLayoutPanel



"""





class Timeline(Animatable,ISealable,IAnimatable,IResource):
 """ Defines a segment of time. """
 def AllocateClock(self,*args):
  """
  AllocateClock(self: Timeline) -> Clock

  

   Creates a System.Windows.Media.Animation.Clock for this System.Windows.Media.Animation.Timeline.

   Returns: A clock for this System.Windows.Media.Animation.Timeline.
  """
  pass
 def Clone(self):
  """
  Clone(self: Timeline) -> Timeline

  

   Creates a modifiable clone of this System.Windows.Media.Animation.Timeline,making deep copies 

    of this object's values.

  

   Returns: A modifiable clone of the current object. The cloned object's System.Windows.Freezable.IsFrozen 

    property is false even if the source's System.Windows.Freezable.IsFrozen property is true.
  """
  pass
 def CloneCore(self,*args):
  """
  CloneCore(self: Freezable,sourceFreezable: Freezable)

   Makes the instance a clone (deep copy) of the specified System.Windows.Freezable using base 

    (non-animated) property values.

  

  

   sourceFreezable: The object to clone.
  """
  pass
 def CloneCurrentValue(self):
  """
  CloneCurrentValue(self: Timeline) -> Timeline

  

   Creates a modifiable clone of this System.Windows.Media.Animation.Timeline object,making deep 

    copies of this object's current values.

  

   Returns: A modifiable clone of the current object. The cloned object's System.Windows.Freezable.IsFrozen 

    property is false even if the source's System.Windows.Freezable.IsFrozen property is true.
  """
  pass
 def CloneCurrentValueCore(self,*args):
  """
  CloneCurrentValueCore(self: Freezable,sourceFreezable: Freezable)

   Makes the instance a modifiable clone (deep copy) of the specified System.Windows.Freezable 

    using current property values.

  

  

   sourceFreezable: The System.Windows.Freezable to be cloned.
  """
  pass
 def CreateClock(self,hasControllableRoot=None):
  """
  CreateClock(self: Timeline,hasControllableRoot: bool) -> Clock

  

   Creates a new System.Windows.Media.Animation.Clock from this 

    System.Windows.Media.Animation.Timeline and specifies whether the new 

    System.Windows.Media.Animation.Clock is controllable. If this 

    System.Windows.Media.Animation.Timeline has children,a tree of clocks is created with this 

    System.Windows.Media.Animation.Timeline as the root.

  

  

   hasControllableRoot: true if the root System.Windows.Media.Animation.Clock returned should return a 

    System.Windows.Media.Animation.ClockController from its 

    System.Windows.Media.Animation.Clock.Controller property so that the 

    System.Windows.Media.Animation.Clock tree can be interactively controlled; otherwise,false.

  

   Returns: A new System.Windows.Media.Animation.Clock constructed from this 

    System.Windows.Media.Animation.Timeline. If this System.Windows.Media.Animation.Timeline is a 

    System.Windows.Media.Animation.TimelineGroup that contains child timelines,a tree of 

    System.Windows.Media.Animation.Clock objects is created with a controllable 

    System.Windows.Media.Animation.Clock created from this System.Windows.Media.Animation.Timeline 

    as the root.

  

  CreateClock(self: Timeline) -> Clock

  

   Creates a new,controllable System.Windows.Media.Animation.Clock from this 

    System.Windows.Media.Animation.Timeline. If this System.Windows.Media.Animation.Timeline has 

    children,a tree of clocks is created with this System.Windows.Media.Animation.Timeline as the 

    root.

  

   Returns: A new,controllable System.Windows.Media.Animation.Clock constructed from this 

    System.Windows.Media.Animation.Timeline. If this System.Windows.Media.Animation.Timeline is a 

    System.Windows.Media.Animation.TimelineGroup that contains child timelines,a tree of 

    System.Windows.Media.Animation.Clock objects is created with a controllable 

    System.Windows.Media.Animation.Clock created from this System.Windows.Media.Animation.Timeline 

    as the root.
  """
  pass
 def CreateInstance(self,*args):
  """
  CreateInstance(self: Freezable) -> Freezable

  

   Initializes a new instance of the System.Windows.Freezable class.

   Returns: The new instance.
  """
  pass
 def CreateInstanceCore(self,*args):
  """
  CreateInstanceCore(self: Freezable) -> Freezable

  

   When implemented in a derived class,creates a new instance of the System.Windows.Freezable 

    derived class.

  

   Returns: The new instance.
  """
  pass
 def FreezeCore(self,*args):
  """
  FreezeCore(self: Timeline,isChecking: bool) -> bool

  

   Makes this System.Windows.Media.Animation.Timeline unmodifiable or determines whether it can be 

    made unmodifiable.

  

  

   isChecking: true to check if this instance can be frozen; false to freeze this instance.

   Returns: If isChecking is true,this method returns true if this instance can be made read-only,or false 

    if it cannot be made read-only. If isChecking is false,this method returns true if this 

    instance is now read-only,or false if it cannot be made read-only,with the side effect of 

    having begun to change the frozen status of this object.
  """
  pass
 def GetAsFrozenCore(self,*args):
  """
  GetAsFrozenCore(self: Timeline,sourceFreezable: Freezable)

   Makes this instance a clone of the specified System.Windows.Media.Animation.Timeline object.

  

   sourceFreezable: The System.Windows.Media.Animation.Timeline instance to clone.
  """
  pass
 def GetCurrentValueAsFrozenCore(self,*args):
  """
  GetCurrentValueAsFrozenCore(self: Timeline,sourceFreezable: Freezable)

   Makes this instance a frozen clone of the specified System.Windows.Media.Animation.Timeline. 

    Resource references,data bindings,and animations are not copied,but their current values are.

  

  

   sourceFreezable: The System.Windows.Media.Animation.Timeline to copy and freeze.
  """
  pass
 @staticmethod
 def GetDesiredFrameRate(timeline):
  """
  GetDesiredFrameRate(timeline: Timeline) -> Nullable[int]

  

   Gets the desired frame rate of the specified System.Windows.Media.Animation.Timeline.

  

   timeline: The timeline from which to retrieve the desired frame rate.

   Returns: The desired frame rate of this timeline. The default value is null.
  """
  pass
 def GetNaturalDuration(self,*args):
  """
  GetNaturalDuration(self: Timeline,clock: Clock) -> Duration

  

   Returns the length of a single iteration of this System.Windows.Media.Animation.Timeline.

  

   clock: The System.Windows.Media.Animation.Clock that was created for this 

    System.Windows.Media.Animation.Timeline.

  

   Returns: The length of a single iteration of this System.Windows.Media.Animation.Timeline,or 

    System.Windows.Duration.Automatic if the natural duration is unknown.
  """
  pass
 def GetNaturalDurationCore(self,*args):
  """
  GetNaturalDurationCore(self: Timeline,clock: Clock) -> Duration

  

   Returns the length of a single iteration of this System.Windows.Media.Animation.Timeline. This 

    method provides the implementation for 

    System.Windows.Media.Animation.Timeline.GetNaturalDuration(System.Windows.Media.Animation.Clock).

  

  

   clock: The System.Windows.Media.Animation.Clock that was created for this 

    System.Windows.Media.Animation.Timeline.

  

   Returns: The length of a single iteration of this System.Windows.Media.Animation.Timeline,or 

    System.Windows.Duration.Automatic if the natural duration is unknown.
  """
  pass
 def OnChanged(self,*args):
  """
  OnChanged(self: Freezable)

   Called when the current System.Windows.Freezable object is modified.
  """
  pass
 def OnFreezablePropertyChanged(self,*args):
  """
  OnFreezablePropertyChanged(self: Freezable,oldValue: DependencyObject,newValue: DependencyObject,property: DependencyProperty)

   This member supports the Windows Presentation Foundation (WPF) infrastructure and is not 

    intended to be used directly from your code.

  

  

   oldValue: The previous value of the data member.

   newValue: The current value of the data member.

   property: The property that changed.

  OnFreezablePropertyChanged(self: Freezable,oldValue: DependencyObject,newValue: DependencyObject)

   Ensures that appropriate context pointers are established for a 

    System.Windows.DependencyObjectType data member that has just been set.

  

  

   oldValue: The previous value of the data member.

   newValue: The current value of the data member.
  """
  pass
 def OnPropertyChanged(self,*args):
  """
  OnPropertyChanged(self: Freezable,e: DependencyPropertyChangedEventArgs)

   Overrides the System.Windows.DependencyObject implementation of 

    System.Windows.DependencyObject.OnPropertyChanged(System.Windows.DependencyPropertyChangedEventAr

    gs) to also invoke any System.Windows.Freezable.Changed handlers in response to a changing 

    dependency property of type System.Windows.Freezable.

  

  

   e: Event data that contains information about which property changed,and its old and new values.
  """
  pass
 def ReadPreamble(self,*args):
  """
  ReadPreamble(self: Freezable)

   Ensures that the System.Windows.Freezable is being accessed from a valid thread. Inheritors of 

    System.Windows.Freezable must call this method at the beginning of any API that reads data 

    members that are not dependency properties.
  """
  pass
 @staticmethod
 def SetDesiredFrameRate(timeline,desiredFrameRate):
  """ SetDesiredFrameRate(timeline: Timeline,desiredFrameRate: Nullable[int]) """
  pass
 def ShouldSerializeProperty(self,*args):
  """
  ShouldSerializeProperty(self: DependencyObject,dp: DependencyProperty) -> bool

  

   Returns a value that indicates whether serialization processes should serialize the value for 

    the provided dependency property.

  

  

   dp: The identifier for the dependency property that should be serialized.

   Returns: true if the dependency property that is supplied should be value-serialized; otherwise,false.
  """
  pass
 def WritePostscript(self,*args):
  """
  WritePostscript(self: Freezable)

   Raises the System.Windows.Freezable.Changed event for the System.Windows.Freezable and invokes 

    its System.Windows.Freezable.OnChanged method. Classes that derive from System.Windows.Freezable 

    should call this method at the end of any API that modifies class members that are not stored as 

    dependency properties.
  """
  pass
 def WritePreamble(self,*args):
  """
  WritePreamble(self: Freezable)

   Verifies that the System.Windows.Freezable is not frozen and that it is being accessed from a 

    valid threading context. System.Windows.Freezable inheritors should call this method at the 

    beginning of any API that writes to data members that are not dependency properties.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,*args): #cannot find CLR constructor
  """
  __new__(cls: type)

  __new__(cls: type,beginTime: Nullable[TimeSpan])

  __new__(cls: type,beginTime: Nullable[TimeSpan],duration: Duration)

  __new__(cls: type,beginTime: Nullable[TimeSpan],duration: Duration,repeatBehavior: RepeatBehavior)
  """
  pass
 AccelerationRatio=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value specifying the percentage of the timeline's System.Windows.Media.Animation.Timeline.Duration spent accelerating the passage of time from zero to its maximum rate.



Get: AccelerationRatio(self: Timeline) -> float



Set: AccelerationRatio(self: Timeline)=value

"""

 AutoReverse=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value that indicates whether the timeline plays in reverse after it completes a forward iteration.



Get: AutoReverse(self: Timeline) -> bool



Set: AutoReverse(self: Timeline)=value

"""

 BeginTime=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the time at which this System.Windows.Media.Animation.Timeline should begin.



Get: BeginTime(self: Timeline) -> Nullable[TimeSpan]



Set: BeginTime(self: Timeline)=value

"""

 DecelerationRatio=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value specifying the percentage of the timeline's System.Windows.Media.Animation.Timeline.Duration spent decelerating the passage of time from its maximum rate to zero.



Get: DecelerationRatio(self: Timeline) -> float



Set: DecelerationRatio(self: Timeline)=value

"""

 Duration=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the length of time for which this timeline plays,not counting repetitions.



Get: Duration(self: Timeline) -> Duration



Set: Duration(self: Timeline)=value

"""

 FillBehavior=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value that specifies how the System.Windows.Media.Animation.Timeline behaves after it reaches the end of its active period.



Get: FillBehavior(self: Timeline) -> FillBehavior



Set: FillBehavior(self: Timeline)=value

"""

 Name=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the name of this System.Windows.Media.Animation.Timeline.



Get: Name(self: Timeline) -> str



Set: Name(self: Timeline)=value

"""

 RepeatBehavior=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the repeating behavior of this timeline.



Get: RepeatBehavior(self: Timeline) -> RepeatBehavior



Set: RepeatBehavior(self: Timeline)=value

"""

 SpeedRatio=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the rate,relative to its parent,at which time progresses for this System.Windows.Media.Animation.Timeline.



Get: SpeedRatio(self: Timeline) -> float



Set: SpeedRatio(self: Timeline)=value

"""


 AccelerationRatioProperty=None
 AutoReverseProperty=None
 BeginTimeProperty=None
 Completed=None
 CurrentGlobalSpeedInvalidated=None
 CurrentStateInvalidated=None
 CurrentTimeInvalidated=None
 DecelerationRatioProperty=None
 DesiredFrameRateProperty=None
 DurationProperty=None
 FillBehaviorProperty=None
 NameProperty=None
 RemoveRequested=None
 RepeatBehaviorProperty=None
 SpeedRatioProperty=None




class ExtenderProvidedPropertyAttribute(Attribute,_Attribute):
 """
 Specifies a property that is offered by an extender provider. This class cannot be inherited.

 

 ExtenderProvidedPropertyAttribute()
 """
 def Equals(self,obj):
  """
  Equals(self: ExtenderProvidedPropertyAttribute,obj: object) -> bool

  

   Determines whether the specified System.Object is equal to the current System.Object.

  

   obj: An System.Object to compare with this instance or null.

   Returns: true if the specified System.Object is equal to the current System.Object; otherwise,false.
  """
  pass
 def GetHashCode(self):
  """
  GetHashCode(self: ExtenderProvidedPropertyAttribute) -> int

  

   Returns the hash code for this instance.

   Returns: A 32-bit signed integer hash code.
  """
  pass
 def IsDefaultAttribute(self):
  """
  IsDefaultAttribute(self: ExtenderProvidedPropertyAttribute) -> bool

  

   Provides an indication whether the value of this instance is the default value for the derived 

    class.

  

   Returns: true if this instance is the default attribute for the class; otherwise,false.
  """
  pass
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==y """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __ne__(self,*args):
  pass
 ExtenderProperty=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the property that is being provided.



Get: ExtenderProperty(self: ExtenderProvidedPropertyAttribute) -> PropertyDescriptor



"""

 Provider=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the extender provider that is providing the property.



Get: Provider(self: ExtenderProvidedPropertyAttribute) -> IExtenderProvider



"""

 ReceiverType=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the type of object that can receive the property.



Get: ReceiverType(self: ExtenderProvidedPropertyAttribute) -> Type



"""





# encoding: utf-8
# module System.Runtime.Versioning calls itself Versioning
# from mscorlib,Version=4.0.0.0,Culture=neutral,PublicKeyToken=b77a5c561934e089,System,Version=4.0.0.0,Culture=neutral,PublicKeyToken=b77a5c561934e089
# by generator 1.145
""" NamespaceTracker represent a CLS namespace. """
# no imports

# no functions
# classes

class CompatibilitySwitch(object):
 # no doc
 @staticmethod
 def GetValue(compatibilitySwitchName):
  """ GetValue(compatibilitySwitchName: str) -> str """
  pass
 @staticmethod
 def IsEnabled(compatibilitySwitchName):
  """ IsEnabled(compatibilitySwitchName: str) -> bool """
  pass
 __all__=[
  'GetValue',
  'IsEnabled',
 ]


class ComponentGuaranteesAttribute(Attribute,_Attribute):
 """
 Defines the compatibility guarantee of a component,type,or type member that may span multiple versions.

 

 ComponentGuaranteesAttribute(guarantees: ComponentGuaranteesOptions)
 """
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,guarantees):
  """ __new__(cls: type,guarantees: ComponentGuaranteesOptions) """
  pass
 Guarantees=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates the guaranteed level of compatibility of a library,type,or type member that spans multiple versions.



Get: Guarantees(self: ComponentGuaranteesAttribute) -> ComponentGuaranteesOptions



"""



class ComponentGuaranteesOptions(Enum,IComparable,IFormattable,IConvertible):
 """
 Describes the compatibility guarantee of a component,type,or type member that may span multiple versions.

 

 enum (flags) ComponentGuaranteesOptions,values: Exchange (1),None (0),SideBySide (4),Stable (2)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 Exchange=None
 None=None
 SideBySide=None
 Stable=None
 value__=None


class FrameworkName(object,IEquatable[FrameworkName]):
 """
 Represents the name of a version of the .NET Framework.

 

 FrameworkName(identifier: str,version: Version)

 FrameworkName(identifier: str,version: Version,profile: str)

 FrameworkName(frameworkName: str)
 """
 def Equals(self,*__args):
  """
  Equals(self: FrameworkName,other: FrameworkName) -> bool

  

   Returns a value that indicates whether this System.Runtime.Versioning.FrameworkName instance 

    represents the same .NET Framework version as a specified 

    System.Runtime.Versioning.FrameworkName instance.

  

  

   other: The object to compare to the current instance.

   Returns: true if every component of the current System.Runtime.Versioning.FrameworkName object matches 

    the corresponding component of other; otherwise,false.

  

  Equals(self: FrameworkName,obj: object) -> bool

  

   Returns a value that indicates whether this System.Runtime.Versioning.FrameworkName instance 

    represents the same .NET Framework version as a specified object.

  

  

   obj: The object to compare to the current instance.

   Returns: true if every component of the current System.Runtime.Versioning.FrameworkName object matches 

    the corresponding component of obj; otherwise,false.
  """
  pass
 def GetHashCode(self):
  """
  GetHashCode(self: FrameworkName) -> int

  

   Returns the hash code for the System.Runtime.Versioning.FrameworkName object.

   Returns: A 32-bit signed integer that represents the hash code of this instance.
  """
  pass
 def ToString(self):
  """
  ToString(self: FrameworkName) -> str

  

   Returns the string representation of this System.Runtime.Versioning.FrameworkName object.

   Returns: A string that represents this System.Runtime.Versioning.FrameworkName object.
  """
  pass
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==y """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,*__args):
  """
  __new__(cls: type,identifier: str,version: Version)

  __new__(cls: type,identifier: str,version: Version,profile: str)

  __new__(cls: type,frameworkName: str)
  """
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __repr__(self,*args):
  """ __repr__(self: object) -> str """
  pass
 def __str__(self,*args):
  pass
 FullName=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the full name of this System.Runtime.Versioning.FrameworkName object.



Get: FullName(self: FrameworkName) -> str



"""

 Identifier=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the identifier of this System.Runtime.Versioning.FrameworkName object.



Get: Identifier(self: FrameworkName) -> str



"""

 Profile=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the profile name of this System.Runtime.Versioning.FrameworkName object.



Get: Profile(self: FrameworkName) -> str



"""

 Version=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the version of this System.Runtime.Versioning.FrameworkName object.



Get: Version(self: FrameworkName) -> Version



"""



class ResourceConsumptionAttribute(Attribute,_Attribute):
 """
 Specifies the resource consumed by the member of a class. This class cannot be inherited.

 

 ResourceConsumptionAttribute(resourceScope: ResourceScope)

 ResourceConsumptionAttribute(resourceScope: ResourceScope,consumptionScope: ResourceScope)
 """
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,resourceScope,consumptionScope=None):
  """
  __new__(cls: type,resourceScope: ResourceScope)

  __new__(cls: type,resourceScope: ResourceScope,consumptionScope: ResourceScope)
  """
  pass
 ConsumptionScope=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the consumption scope for this member.



Get: ConsumptionScope(self: ResourceConsumptionAttribute) -> ResourceScope



"""

 ResourceScope=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the resource scope for the consumed resource.



Get: ResourceScope(self: ResourceConsumptionAttribute) -> ResourceScope



"""



class ResourceExposureAttribute(Attribute,_Attribute):
 """
 Specifies the resource exposure for a member of a class. This class cannot be inherited.

 

 ResourceExposureAttribute(exposureLevel: ResourceScope)
 """
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,exposureLevel):
  """ __new__(cls: type,exposureLevel: ResourceScope) """
  pass
 ResourceExposureLevel=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the resource exposure scope.



Get: ResourceExposureLevel(self: ResourceExposureAttribute) -> ResourceScope



"""



class ResourceScope(Enum,IComparable,IFormattable,IConvertible):
 """
 Identifies the scope of a sharable resource.

 

 enum (flags) ResourceScope,values: AppDomain (4),Assembly (32),Library (8),Machine (1),None (0),Private (16),Process (2)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 AppDomain=None
 Assembly=None
 Library=None
 Machine=None
 None=None
 Private=None
 Process=None
 value__=None


class TargetFrameworkAttribute(Attribute,_Attribute):
 """
 Identifies the version of the .NET Framework that a particular assembly was compiled against.

 

 TargetFrameworkAttribute(frameworkName: str)
 """
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,frameworkName):
  """ __new__(cls: type,frameworkName: str) """
  pass
 FrameworkDisplayName=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the display name of the .NET Framework version against which an assembly was built.



Get: FrameworkDisplayName(self: TargetFrameworkAttribute) -> str



Set: FrameworkDisplayName(self: TargetFrameworkAttribute)=value

"""

 FrameworkName=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the name of the .NET Framework version against which a particular assembly was compiled.



Get: FrameworkName(self: TargetFrameworkAttribute) -> str



"""



class VersioningHelper(object):
 """ Provides methods to aid developers in writing version-safe code. This class cannot be inherited. """
 @staticmethod
 def MakeVersionSafeName(name,from,to,type=None):
  """
  MakeVersionSafeName(name: str,from: ResourceScope,to: ResourceScope) -> str

  

   Returns a version-safe name based on the specified resource name and the intended resource 

    consumption source.

  

  

   name: The name of the resource.

   from: The scope of the resource.

   to: The desired resource consumption scope.

   Returns: A version-safe name.

  MakeVersionSafeName(name: str,from: ResourceScope,to: ResourceScope,type: Type) -> str

  

   Returns a version-safe name based on the specified resource name,the intended resource 

    consumption scope,and the type using the resource.

  

  

   name: The name of the resource.

   from: The beginning of the scope range.

   to: The end of the scope range.

   type: The System.Type of the resource.

   Returns: A version-safe name.
  """
  pass
 __all__=[
  'MakeVersionSafeName',
 ]





class LabelEditEventArgs(EventArgs):
 """
 Provides data for the System.Windows.Forms.ListView.BeforeLabelEdit and System.Windows.Forms.ListView.AfterLabelEdit events.

 

 LabelEditEventArgs(item: int)

 LabelEditEventArgs(item: int,label: str)
 """
 def __getitem__(self,*args):
  """ x.__getitem__(y) <==> x[y] """
  pass
 @staticmethod
 def __new__(self,item,label=None):
  """
  __new__(cls: type,item: int)

  __new__(cls: type,item: int,label: str)
  """
  pass
 CancelEdit=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value indicating whether changes made to the label of the System.Windows.Forms.ListViewItem should be canceled.



Get: CancelEdit(self: LabelEditEventArgs) -> bool



Set: CancelEdit(self: LabelEditEventArgs)=value

"""

 Item=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the zero-based index of the System.Windows.Forms.ListViewItem containing the label to edit.



Get: Item(self: LabelEditEventArgs) -> int



"""

 Label=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the new text assigned to the label of the System.Windows.Forms.ListViewItem.



Get: Label(self: LabelEditEventArgs) -> str



"""





class FileOptions(Enum,IComparable,IFormattable,IConvertible):
 """
 Represents additional options for creating a System.IO.FileStream object.

 

 enum (flags) FileOptions,values: Asynchronous (1073741824),DeleteOnClose (67108864),Encrypted (16384),None (0),RandomAccess (268435456),SequentialScan (134217728),WriteThrough (-2147483648)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 Asynchronous=None
 DeleteOnClose=None
 Encrypted=None
 None=None
 RandomAccess=None
 SequentialScan=None
 value__=None
 WriteThrough=None




class HandleCollector(object):
 """
 Tracks outstanding handles and forces a garbage collection when the specified threshold is reached.

 

 HandleCollector(name: str,initialThreshold: int)

 HandleCollector(name: str,initialThreshold: int,maximumThreshold: int)
 """
 def Add(self):
  """
  Add(self: HandleCollector)

   Increments the current handle count.
  """
  pass
 def Remove(self):
  """
  Remove(self: HandleCollector)

   Decrements the current handle count.
  """
  pass
 def __add__(self,*args):
  """ x.__add__(y) <==> x+y """
  pass
 @staticmethod
 def __new__(self,name,initialThreshold,maximumThreshold=None):
  """
  __new__(cls: type,name: str,initialThreshold: int)

  __new__(cls: type,name: str,initialThreshold: int,maximumThreshold: int)
  """
  pass
 Count=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the number of handles collected.



Get: Count(self: HandleCollector) -> int



"""

 InitialThreshold=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that specifies the point at which collections should begin.



Get: InitialThreshold(self: HandleCollector) -> int



"""

 MaximumThreshold=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that specifies the point at which collections must occur.



Get: MaximumThreshold(self: HandleCollector) -> int



"""

 Name=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the name of a System.Runtime.InteropServices.HandleCollector object.



Get: Name(self: HandleCollector) -> str



"""





from typing import Tuple, Set, Iterable, List


class SteelElementProperties(APIObject):
    @property
    def UniqueID(self) -> Guid: ...
    @property
    def IsValidObject(self) -> bool: ...
    def GetSteelElementProperties(pElement: Element) -> SteelElementProperties: ...
    def GetFabricationUniqueID(aDoc: Document, reference: Reference) -> Guid: ...
    def GetReference(aDoc: Document, guid: Guid) -> Reference: ...
    def AddFabricationInformationForRevitElements(aDoc: Document, elementIds: List[ElementId]) -> List[ElementId]: ...



class ISite(IServiceProvider):
 """ Provides functionality required by sites. """
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 Component=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the component associated with the System.ComponentModel.ISite when implemented by a class.



Get: Component(self: ISite) -> IComponent



"""

 Container=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the System.ComponentModel.IContainer associated with the System.ComponentModel.ISite when implemented by a class.



Get: Container(self: ISite) -> IContainer



"""

 DesignMode=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Determines whether the component is in design mode when implemented by a class.



Get: DesignMode(self: ISite) -> bool



"""

 Name=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the name of the component associated with the System.ComponentModel.ISite when implemented by a class.



Get: Name(self: ISite) -> str



Set: Name(self: ISite)=value

"""





class RefreshPropertiesAttribute(Attribute,_Attribute):
 """
 Indicates that the property grid should refresh when the associated property value changes. This class cannot be inherited.

 

 RefreshPropertiesAttribute(refresh: RefreshProperties)
 """
 def Equals(self,value):
  """
  Equals(self: RefreshPropertiesAttribute,value: object) -> bool

  

   Overrides the object's erload:System.Object.Equals method.

  

   value: The object to test for equality.

   Returns: true if the specified object is the same; otherwise,false.
  """
  pass
 def GetHashCode(self):
  """
  GetHashCode(self: RefreshPropertiesAttribute) -> int

  

   Returns the hash code for this object.

   Returns: The hash code for the object that the attribute belongs to.
  """
  pass
 def IsDefaultAttribute(self):
  """
  IsDefaultAttribute(self: RefreshPropertiesAttribute) -> bool

  

   Gets a value indicating whether the current value of the attribute is the default value for the 

    attribute.

  

   Returns: true if the current value of the attribute is the default; otherwise,false.
  """
  pass
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==y """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,refresh):
  """ __new__(cls: type,refresh: RefreshProperties) """
  pass
 def __ne__(self,*args):
  pass
 RefreshProperties=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the refresh properties for the member.



Get: RefreshProperties(self: RefreshPropertiesAttribute) -> RefreshProperties



"""


 All=None
 Default=None
 Repaint=None




class IComNativeDescriptorHandler:
 """ Provides a top-level mapping layer between a COM object and a System.ComponentModel.TypeDescriptor. """
 def GetAttributes(self,component):
  """
  GetAttributes(self: IComNativeDescriptorHandler,component: object) -> AttributeCollection

  

   Gets the attributes for the specified component.

  

   component: The component to get attributes for.

   Returns: A collection of attributes for component.
  """
  pass
 def GetClassName(self,component):
  """
  GetClassName(self: IComNativeDescriptorHandler,component: object) -> str

  

   Gets the class name for the specified component.

  

   component: The component to get the class name for.

   Returns: The name of the class that corresponds with component.
  """
  pass
 def GetConverter(self,component):
  """
  GetConverter(self: IComNativeDescriptorHandler,component: object) -> TypeConverter

  

   Gets the type converter for the specified component.

  

   component: The component to get the System.ComponentModel.TypeConverter for.

   Returns: The System.ComponentModel.TypeConverter for component.
  """
  pass
 def GetDefaultEvent(self,component):
  """
  GetDefaultEvent(self: IComNativeDescriptorHandler,component: object) -> EventDescriptor

  

   Gets the default event for the specified component.

  

   component: The component to get the default event for.

   Returns: An System.ComponentModel.EventDescriptor that represents component's default event.
  """
  pass
 def GetDefaultProperty(self,component):
  """
  GetDefaultProperty(self: IComNativeDescriptorHandler,component: object) -> PropertyDescriptor

  

   Gets the default property for the specified component.

  

   component: The component to get the default property for.

   Returns: A System.ComponentModel.PropertyDescriptor that represents component's default property.
  """
  pass
 def GetEditor(self,component,baseEditorType):
  """
  GetEditor(self: IComNativeDescriptorHandler,component: object,baseEditorType: Type) -> object

  

   Gets the editor for the specified component.

  

   component: The component to get the editor for.

   baseEditorType: The base type of the editor for component.

   Returns: The editor for component.
  """
  pass
 def GetEvents(self,component,attributes=None):
  """
  GetEvents(self: IComNativeDescriptorHandler,component: object,attributes: Array[Attribute]) -> EventDescriptorCollection

  

   Gets the events with the specified attributes for the specified component.

  

   component: The component to get events for.

   attributes: The attributes used to filter events.

   Returns: A collection of event descriptors for component.

  GetEvents(self: IComNativeDescriptorHandler,component: object) -> EventDescriptorCollection

  

   Gets the events for the specified component.

  

   component: The component to get events for.

   Returns: A collection of event descriptors for component.
  """
  pass
 def GetName(self,component):
  """
  GetName(self: IComNativeDescriptorHandler,component: object) -> str

  

   Gets the name of the specified component.

  

   component: The component to get the name of.

   Returns: The name of component.
  """
  pass
 def GetProperties(self,component,attributes):
  """
  GetProperties(self: IComNativeDescriptorHandler,component: object,attributes: Array[Attribute]) -> PropertyDescriptorCollection

  

   Gets the properties with the specified attributes for the specified component.

  

   component: The component to get events for.

   attributes: The attributes used to filter properties.

   Returns: A collection of property descriptors for component.
  """
  pass
 def GetPropertyValue(self,component,*__args):
  """
  GetPropertyValue(self: IComNativeDescriptorHandler,component: object,dispid: int,success: bool) -> (object,bool)

  

   Gets the value of the property that has the specified dispatch identifier.

  

   component: The object to which the property belongs.

   dispid: The dispatch identifier.

   success: A System.Boolean,passed by reference,that represents whether the property was retrieved.

   Returns: The value of the property that has the specified dispatch identifier.

  GetPropertyValue(self: IComNativeDescriptorHandler,component: object,propertyName: str,success: bool) -> (object,bool)

  

   Gets the value of the property that has the specified name.

  

   component: The object to which the property belongs.

   propertyName: The name of the property.

   success: A System.Boolean,passed by reference,that represents whether the property was retrieved.

   Returns: The value of the property that has the specified name.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass



class FileInfo(FileSystemInfo,ISerializable):
 """
 Provides properties and instance methods for the creation,copying,deletion,moving,and opening of files,and aids in the creation of System.IO.FileStream objects. This class cannot be inherited.

 

 FileInfo(fileName: str)
 """
 def AppendText(self):
  """
  AppendText(self: FileInfo) -> StreamWriter

  

   Creates a System.IO.StreamWriter that appends text to the file represented by this instance of 

    the System.IO.FileInfo.

  

   Returns: A new StreamWriter.
  """
  pass
 def CopyTo(self,destFileName,overwrite=None):
  """
  CopyTo(self: FileInfo,destFileName: str,overwrite: bool) -> FileInfo

  

   Copies an existing file to a new file,allowing the overwriting of an existing file.

  

   destFileName: The name of the new file to copy to.

   overwrite: true to allow an existing file to be overwritten; otherwise,false.

   Returns: A new file,or an overwrite of an existing file if overwrite is true. If the file exists and 

    overwrite is false,an System.IO.IOException is thrown.

  

  CopyTo(self: FileInfo,destFileName: str) -> FileInfo

  

   Copies an existing file to a new file,disallowing the overwriting of an existing file.

  

   destFileName: The name of the new file to copy to.

   Returns: A new file with a fully qualified path.
  """
  pass
 def Create(self):
  """
  Create(self: FileInfo) -> FileStream

  

   Creates a file.

   Returns: A new file.
  """
  pass
 def CreateText(self):
  """
  CreateText(self: FileInfo) -> StreamWriter

  

   Creates a System.IO.StreamWriter that writes a new text file.

   Returns: A new StreamWriter.
  """
  pass
 def Decrypt(self):
  """
  Decrypt(self: FileInfo)

   Decrypts a file that was encrypted by the current account using the System.IO.FileInfo.Encrypt 

    method.
  """
  pass
 def Delete(self):
  """
  Delete(self: FileInfo)

   Permanently deletes a file.
  """
  pass
 def Encrypt(self):
  """
  Encrypt(self: FileInfo)

   Encrypts a file so that only the account used to encrypt the file can decrypt it.
  """
  pass
 def GetAccessControl(self,includeSections=None):
  """
  GetAccessControl(self: FileInfo,includeSections: AccessControlSections) -> FileSecurity

  

   Gets a System.Security.AccessControl.FileSecurity object that encapsulates the specified type of 

    access control list (ACL) entries for the file described by the current System.IO.FileInfo 

    object.

  

  

   includeSections: One of the System.Security.AccessControl.AccessControlSections values that specifies which group 

    of access control entries to retrieve.

  

   Returns: A System.Security.AccessControl.FileSecurity object that encapsulates the access control rules 

    for the current file.

  

  GetAccessControl(self: FileInfo) -> FileSecurity

  

   Gets a System.Security.AccessControl.FileSecurity object that encapsulates the access control 

    list (ACL) entries for the file described by the current System.IO.FileInfo object.

  

   Returns: A System.Security.AccessControl.FileSecurity object that encapsulates the access control rules 

    for the current file.
  """
  pass
 def MemberwiseClone(self,*args):
  """
  MemberwiseClone(self: MarshalByRefObject,cloneIdentity: bool) -> MarshalByRefObject

  

   Creates a shallow copy of the current System.MarshalByRefObject object.

  

   cloneIdentity: false to delete the current System.MarshalByRefObject object's identity,which will cause the 

    object to be assigned a new identity when it is marshaled across a remoting boundary. A value of 

    false is usually appropriate. true to copy the current System.MarshalByRefObject object's 

    identity to its clone,which will cause remoting client calls to be routed to the remote server 

    object.

  

   Returns: A shallow copy of the current System.MarshalByRefObject object.

  MemberwiseClone(self: object) -> object

  

   Creates a shallow copy of the current System.Object.

   Returns: A shallow copy of the current System.Object.
  """
  pass
 def MoveTo(self,destFileName):
  """
  MoveTo(self: FileInfo,destFileName: str)

   Moves a specified file to a new location,providing the option to specify a new file name.

  

   destFileName: The path to move the file to,which can specify a different file name.
  """
  pass
 def Open(self,mode,access=None,share=None):
  """
  Open(self: FileInfo,mode: FileMode,access: FileAccess,share: FileShare) -> FileStream

  

   Opens a file in the specified mode with read,write,or read/write access and the specified 

    sharing option.

  

  

   mode: A System.IO.FileMode constant specifying the mode (for example,Open or Append) in which to open 

    the file.

  

   access: A System.IO.FileAccess constant specifying whether to open the file with Read,Write,or 

    ReadWrite file access.

  

   share: A System.IO.FileShare constant specifying the type of access other FileStream objects have to 

    this file.

  

   Returns: A System.IO.FileStream object opened with the specified mode,access,and sharing options.

  Open(self: FileInfo,mode: FileMode,access: FileAccess) -> FileStream

  

   Opens a file in the specified mode with read,write,or read/write access.

  

   mode: A System.IO.FileMode constant specifying the mode (for example,Open or Append) in which to open 

    the file.

  

   access: A System.IO.FileAccess constant specifying whether to open the file with Read,Write,or 

    ReadWrite file access.

  

   Returns: A System.IO.FileStream object opened in the specified mode and access,and unshared.

  Open(self: FileInfo,mode: FileMode) -> FileStream

  

   Opens a file in the specified mode.

  

   mode: A System.IO.FileMode constant specifying the mode (for example,Open or Append) in which to open 

    the file.

  

   Returns: A file opened in the specified mode,with read/write access and unshared.
  """
  pass
 def OpenRead(self):
  """
  OpenRead(self: FileInfo) -> FileStream

  

   Creates a read-only System.IO.FileStream.

   Returns: A new read-only System.IO.FileStream object.
  """
  pass
 def OpenText(self):
  """
  OpenText(self: FileInfo) -> StreamReader

  

   Creates a System.IO.StreamReader with UTF8 encoding that reads from an existing text file.

   Returns: A new StreamReader with UTF8 encoding.
  """
  pass
 def OpenWrite(self):
  """
  OpenWrite(self: FileInfo) -> FileStream

  

   Creates a write-only System.IO.FileStream.

   Returns: A write-only unshared System.IO.FileStream object for a new or existing file.
  """
  pass
 def Replace(self,destinationFileName,destinationBackupFileName,ignoreMetadataErrors=None):
  """
  Replace(self: FileInfo,destinationFileName: str,destinationBackupFileName: str,ignoreMetadataErrors: bool) -> FileInfo

  

   Replaces the contents of a specified file with the file described by the current 

    System.IO.FileInfo object,deleting the original file,and creating a backup of the replaced 

    file.  Also specifies whether to ignore merge errors.

  

  

   destinationFileName: The name of a file to replace with the current file.

   destinationBackupFileName: The name of a file with which to create a backup of the file described by the destFileName 

    parameter.

  

   ignoreMetadataErrors: true to ignore merge errors (such as attributes and ACLs) from the replaced file to the 

    replacement file; otherwise false.

  

   Returns: A System.IO.FileInfo object that encapsulates information about the file described by the 

    destFileName parameter.

  

  Replace(self: FileInfo,destinationFileName: str,destinationBackupFileName: str) -> FileInfo

  

   Replaces the contents of a specified file with the file described by the current 

    System.IO.FileInfo object,deleting the original file,and creating a backup of the replaced 

    file.

  

  

   destinationFileName: The name of a file to replace with the current file.

   destinationBackupFileName: The name of a file with which to create a backup of the file described by the destFileName 

    parameter.

  

   Returns: A System.IO.FileInfo object that encapsulates information about the file described by the 

    destFileName parameter.
  """
  pass
 def SetAccessControl(self,fileSecurity):
  """
  SetAccessControl(self: FileInfo,fileSecurity: FileSecurity)

   Applies access control list (ACL) entries described by a 

    System.Security.AccessControl.FileSecurity object to the file described by the current 

    System.IO.FileInfo object.

  

  

   fileSecurity: A System.Security.AccessControl.FileSecurity object that describes an access control list (ACL) 

    entry to apply to the current file.
  """
  pass
 def ToString(self):
  """
  ToString(self: FileInfo) -> str

  

   Returns the path as a string.

   Returns: A string representing the path.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,fileName):
  """ __new__(cls: type,fileName: str) """
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 Directory=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets an instance of the parent directory.



Get: Directory(self: FileInfo) -> DirectoryInfo



"""

 DirectoryName=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a string representing the directory's full path.



Get: DirectoryName(self: FileInfo) -> str



"""

 Exists=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value indicating whether a file exists.



Get: Exists(self: FileInfo) -> bool



"""

 IsReadOnly=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value that determines if the current file is read only.



Get: IsReadOnly(self: FileInfo) -> bool



Set: IsReadOnly(self: FileInfo)=value

"""

 Length=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the size,in bytes,of the current file.



Get: Length(self: FileInfo) -> Int64



"""

 Name=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the name of the file.



Get: Name(self: FileInfo) -> str



"""


 FullPath=None
 OriginalPath=None




"""Prepare and compile python script types."""
import json
from pyrevit.coreutils import logger

import pyrevit.extensions as exts
from pyrevit.runtime import bundletypemaker


#pylint: disable=W0703,C0302,C0103
mlogger = logger.get_logger(__name__)


def create_executor_type(extension, module_builder, cmd_component):
    """Create the dotnet type for the executor.

    Args:
        extension (pyrevit.extensions.components.Extension): pyRevit extension
        module_builder (ModuleBuilder): module builder
        cmd_component (pyrevit.extensions.genericcomps.GenericUICommand):
            command
    """
    engine_configs = json.dumps({
        exts.MDATA_ENGINE_CLEAN:
            cmd_component.requires_clean_engine,

        exts.MDATA_ENGINE_DYNAMO_AUTOMATE:
            cmd_component.requires_mainthread_engine,

        exts.MDATA_ENGINE_DYNAMO_PATH:
            cmd_component.dynamo_path,

        # exts.MDATA_ENGINE_DYNAMO_PATH_EXEC:
        #     cmd_component.dynamo_path_exec,

        exts.MDATA_ENGINE_DYNAMO_PATH_CHECK_EXIST:
            cmd_component.dynamo_path_check_existing,

        exts.MDATA_ENGINE_DYNAMO_FORCE_MANUAL_RUN:
            cmd_component.dynamo_force_manual_run,

        exts.MDATA_ENGINE_DYNAMO_MODEL_NODES_INFO:
            cmd_component.dynamo_model_nodes_info,
    })

    bundletypemaker.create_executor_type(
        extension,
        module_builder,
        cmd_component,
        eng_cfgs=engine_configs
        )



"""
.NET imports

This module ensures most commonly used .NET classes are loaded for you.for

>>> from rpw.utils.dotnet import List, Enum, Process

"""

import sys

from rpw.utils.logger import logger
from rpw.utils.sphinx_compat import MockImporter

# Attempt to Import clr
try:
    import clr
except ImportError:
    # Running Sphinx. Import MockImporter
    logger.warning('Error Importing CLR. Loading Mock Importer')
    sys.meta_path.append(MockImporter())

################
# .NET IMPORTS #
################

import clr
clr.AddReference('System')                 # Enum, Diagnostics
clr.AddReference('System.Collections')     # List

# Core Imports
from System import Enum
from System.Collections.Generic import List
from System.Diagnostics import Process



class CALLCONV(Enum,IComparable,IFormattable,IConvertible):
 """
 Use System.Runtime.InteropServices.ComTypes.CALLCONV instead.

 

 enum CALLCONV,values: CC_CDECL (1),CC_MACPASCAL (3),CC_MAX (9),CC_MPWCDECL (7),CC_MPWPASCAL (8),CC_MSCPASCAL (2),CC_PASCAL (2),CC_RESERVED (5),CC_STDCALL (4),CC_SYSCALL (6)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 CC_CDECL=None
 CC_MACPASCAL=None
 CC_MAX=None
 CC_MPWCDECL=None
 CC_MPWPASCAL=None
 CC_MSCPASCAL=None
 CC_PASCAL=None
 CC_RESERVED=None
 CC_STDCALL=None
 CC_SYSCALL=None
 value__=None




""" Collection of Class Mixins """

import rpw
# from rpw import revit, db, DB # Fixes Circular Import
from rpw.exceptions import RpwCoerceError
from rpw.utils.logger import deprecate_warning


class ByNameCollectMixin():

    """ Adds name, by_name(), and by_name_or_element_ref() methods.
    This is for class inheritance only, used to reduce duplication
    """

    @property
    def name(self):
        """ Returns object's Name attribute """
        return self._revit_object.Name

    @classmethod
    def by_name(cls, name):
        """
        Mixin to provide instantiating by a name for classes that are
        collectible. This is a mixin so specifi usage will vary for each for.
        This method will call the :any:`rpw.db.Element.collect`
        method of the class, and return the first element with a
        matching ``.name`` property.

        >>> LinePatternElement.by_name('Dash')
        <rpw:LinePatternElement name:Dash>

        >>> FillPatternElement.by_name('Solid')
        <rpw:FillPatternElement name:Solid>

        """
        e = cls.collect(where=lambda e: e.name.lower() == name.lower()).get_first()
        if e:
            return e
        raise RpwCoerceError('by_name({})'.format(name), cls)

    @classmethod
    def by_name_or_element_ref(cls, reference):
        """
        Mixin for collectible elements.
        This is to help cast elements from name, elemente, or element_id
        """
        if isinstance(reference, str):
            return cls.by_name(reference)
        elif isinstance(reference, rpw.DB.ElementId):
            return rpw.db.Element.from_id(reference)
        else:
            return cls(reference)



class CategoryMixin():

    """ Adds category and get_category methods.
    """

    @property
    def _category(self):
        """
        Default Category Access Parameter. Overwrite on wrapper as needed.
        See Family Wrapper for an example.
        """
        return self._revit_object.Category

    @property
    def category(self):
        """ Wrapped ``DB.Category`` """
        deprecate_warning('.category', 'get_category()')
        return rpw.db.Category(self._category)

    def get_category(self, wrapped=True):
        """ Wrapped ``DB.Category``"""
        return rpw.db.Category(self._category) if wrapped else self._category



from typing import Tuple, Set, Iterable, List


class IFCImportOptions:
    def __init__(self): ...
    @property
    def Intent(self) -> IFCImportIntent: ...
    @Intent.setter
    def Intent(self, Intent: IFCImportIntent) -> None: ...
    @property
    def AutoJoin(self) -> bool: ...
    @AutoJoin.setter
    def AutoJoin(self, AutoJoin: bool) -> None: ...
    @property
    def AutocorrectOffAxisLines(self) -> bool: ...
    @AutocorrectOffAxisLines.setter
    def AutocorrectOffAxisLines(self, AutocorrectOffAxisLines: bool) -> None: ...
    @property
    def Action(self) -> IFCImportAction: ...
    @Action.setter
    def Action(self, Action: IFCImportAction) -> None: ...
    @property
    def ForceImport(self) -> bool: ...
    @ForceImport.setter
    def ForceImport(self, ForceImport: bool) -> None: ...
    @property
    def CreateLinkInstanceOnly(self) -> bool: ...
    @CreateLinkInstanceOnly.setter
    def CreateLinkInstanceOnly(self, CreateLinkInstanceOnly: bool) -> None: ...
    @property
    def RevitLinkFileName(self) -> str: ...
    @RevitLinkFileName.setter
    def RevitLinkFileName(self, RevitLinkFileName: str) -> None: ...
    @property
    def IsValidObject(self) -> bool: ...
    def GetConversionData(self) -> LinkConversionData: ...
    def GetExtraOptions(self) -> IDictionary: ...
    def SetExtraOptions(self, options: IDictionary) -> None: ...
    def Dispose(self) -> None: ...


class IFCImportIntent:
    Parametric = 0
    Reference = 1


class IFCImportAction:
    Open = 0
    Link = 1



"""The loader module manages the workflow of loading a new pyRevit session.

Its main purpose is to orchestrate the process of finding pyRevit extensions,
creating dll assemblies for them, and creating a user interface
in the host application.

Everything starts from `sessionmgr.load_session()` function...

The only public function is `load_session()` that loads a new session.
Everything else is private.
"""
import sys
from collections import namedtuple

from pyrevit import EXEC_PARAMS, HOST_APP
from pyrevit import MAIN_LIB_DIR, MISC_LIB_DIR
from pyrevit import framework
from pyrevit.coreutils import Timer
from pyrevit.coreutils import assmutils
from pyrevit.coreutils import envvars
from pyrevit.coreutils import appdata
from pyrevit.coreutils import logger
from pyrevit.coreutils import applocales
from pyrevit.loader import sessioninfo
from pyrevit.loader import asmmaker
from pyrevit.loader import uimaker
from pyrevit.loader import hooks
from pyrevit.userconfig import user_config
from pyrevit.extensions import extensionmgr
from pyrevit.versionmgr import updater
from pyrevit.versionmgr import upgrade
from pyrevit import telemetry
from pyrevit import routes
# import the runtime first to get all the c-sharp code to compile
from pyrevit import runtime
from pyrevit.runtime import types as runtime_types
# now load the rest of module that could depend on the compiled runtime
from pyrevit import output

from pyrevit import DB, UI, revit


#pylint: disable=W0703,C0302,C0103,no-member
mlogger = logger.get_logger(__name__)


AssembledExtension = namedtuple('AssembledExtension', ['ext', 'assm'])


def _clear_running_engines():
    # clear the cached engines
    try:
        my_output = output.get_output()
        if my_output:
            my_output.close_others(all_open_outputs=True)

        runtime_types.ScriptEngineManager.ClearEngines(
            excludeEngine=EXEC_PARAMS.engine_id
            )
        return True
    except AttributeError:
        return False


def _setup_output():
    # create output window and assign handle
    out_window = runtime.types.ScriptConsole()
    runtime_info = sessioninfo.get_runtime_info()
    out_window.AppVersion = '{}:{}:{}'.format(
        runtime_info.pyrevit_version,
        int(runtime_info.engine_version),
        runtime_info.host_version
        )

    # create output stream and set stdout to it
    # we're not opening the output window here.
    # The output stream will open the window if anything is being printed.
    outstr = runtime.types.ScriptIO(out_window)
    sys.stdout = outstr
    # sys.stderr = outstr
    stdout_hndlr = logger.get_stdout_hndlr()
    stdout_hndlr.stream = outstr

    return out_window


def _cleanup_output():
    sys.stdout = None
    stdout_hndlr = logger.get_stdout_hndlr()
    stdout_hndlr.stream = None


# -----------------------------------------------------------------------------
# Functions related to creating/loading a new pyRevit session
# -----------------------------------------------------------------------------
def _check_autoupdate_inprogress():
    return envvars.get_pyrevit_env_var(envvars.AUTOUPDATING_ENVVAR)


def _set_autoupdate_inprogress(state):
    envvars.set_pyrevit_env_var(envvars.AUTOUPDATING_ENVVAR, state)


def _perform_onsessionloadstart_ops():
    # clear the cached engines
    if not _clear_running_engines():
        mlogger.debug('No Engine Manager exists...')

    # check for updates
    if user_config.auto_update \
            and not _check_autoupdate_inprogress():
        mlogger.info('Auto-update is active. Attempting update...')
        _set_autoupdate_inprogress(True)
        updater.update_pyrevit()
        _set_autoupdate_inprogress(False)

    # once pre-load is complete, report environment conditions
    uuid_str = sessioninfo.new_session_uuid()
    sessioninfo.report_env()

    # reset the list of assemblies loaded under pyRevit session
    sessioninfo.set_loaded_pyrevit_assemblies([])

    # init executor
    runtime_types.ScriptExecutor.Initialize()

    # init routes
    routes.init()

    # asking telemetry module to setup the telemetry system
    # (active or not active)
    telemetry.setup_telemetry(uuid_str)

    # apply Upgrades
    upgrade.upgrade_existing_pyrevit()

    # setup hooks
    hooks.setup_hooks()


def _perform_onsessionloadcomplete_ops():
    # cleanup old assembly files.
    asmmaker.cleanup_assembly_files()

    # clean up temp app files between sessions.
    appdata.cleanup_appdata_folder()

    # activate hooks now
    hooks.activate()

    # activate internal handlers
    # toggle doc colorizer
    revit.tabs.init_doc_colorizer(user_config)

    # activate runtime routes server
    if user_config.routes_server:
        routes.active_routes_api()
        active_server = routes.activate_server()
        if active_server:
            mlogger.info(str(active_server))
        else:
            mlogger.error('Routes servers failed activation')


def _new_session():
    """Create an assembly and UI for each installed UI extensions."""
    assembled_exts = []
    # get all installed ui extensions
    for ui_ext in extensionmgr.get_installed_ui_extensions():
        # configure extension components for metadata
        # e.g. liquid templates like {{author}}
        ui_ext.configure()

        # collect all module references from extensions
        ui_ext_modules = []
        # FIXME: currently dlls inside bin/ are not pre-loaded since
        # this will lock them by Revit. Maybe all dlls should be loaded
        # from memory (read binary and load assembly)?
        # ui_ext_modules.extend(ui_ext.get_extension_modules())
        ui_ext_modules.extend(ui_ext.get_command_modules())
        # make sure they are all loaded
        assmutils.load_asm_files(ui_ext_modules)
        # and update env information
        sessioninfo.update_loaded_pyrevit_referenced_modules(ui_ext_modules)

        # create a dll assembly and get assembly info
        ext_asm_info = asmmaker.create_assembly(ui_ext)
        if not ext_asm_info:
            mlogger.critical('Failed to create assembly for: %s', ui_ext)
            continue
        else:
            mlogger.info('Extension assembly created: %s', ui_ext.name)

        assembled_exts.append(
            AssembledExtension(ext=ui_ext, assm=ext_asm_info)
        )

    # add names of the created assemblies to the session info
    sessioninfo.set_loaded_pyrevit_assemblies(
        [x.assm.name for x in assembled_exts]
    )

    # run startup scripts for this ui extension, if any
    for assm_ext in assembled_exts:
        if assm_ext.ext.startup_script:
            # build syspaths for the startup script
            sys_paths = [assm_ext.ext.directory]
            if assm_ext.ext.library_path:
                sys_paths.insert(0, assm_ext.ext.library_path)

            mlogger.info('Running startup tasks for %s', assm_ext.ext.name)
            mlogger.debug('Executing startup script for extension: %s',
                          assm_ext.ext.name)

            # now run
            execute_extension_startup_script(
                assm_ext.ext.startup_script,
                assm_ext.ext.name,
                sys_paths=sys_paths
                )

    # register extension hooks
    for assm_ext in assembled_exts:
        hooks.register_hooks(assm_ext.ext)

    # update/create ui (needs the assembly to link button actions
    # to commands saved in the dll)
    for assm_ext in assembled_exts:
        uimaker.update_pyrevit_ui(
            assm_ext.ext,
            assm_ext.assm,
            user_config.load_beta
        )
        mlogger.info('UI created for extension: %s', assm_ext.ext.name)

    # re-sort the ui elements
    for assm_ext in assembled_exts:
        uimaker.sort_pyrevit_ui(assm_ext.ext)

    # cleanup existing UI. This is primarily for cleanups after reloading
    uimaker.cleanup_pyrevit_ui()

    # reflow the ui if requested, depending on the language direction
    if user_config.respect_language_direction:
        current_applocale = applocales.get_current_applocale()
        uimaker.reflow_pyrevit_ui(direction=current_applocale.lang_dir)
    else:
        uimaker.reflow_pyrevit_ui()


def load_session():
    """Handles loading/reloading of the pyRevit addin and extensions.

    To create a proper ui, pyRevit extensions needs to be properly parsed and
    a dll assembly needs to be created. This function handles these tasks
    through interactions with .extensions, .loader.asmmaker, and .loader.uimaker.

    Examples:
        ```python
        from pyrevit.loader.sessionmgr import load_session
        load_session()     # start loading a new pyRevit session
        ```

    Returns:
        (str): sesion uuid
    """
    # setup runtime environment variables
    sessioninfo.setup_runtime_vars()

    # the loader dll addon, does not create an output window
    # if an output window is not provided, create one
    if EXEC_PARAMS.first_load:
        output_window = _setup_output()
    else:
        from pyrevit import script
        output_window = script.get_output()

    # initialize timer to measure load time
    timer = Timer()

    # perform pre-load tasks
    _perform_onsessionloadstart_ops()

    # create a new session
    _new_session()

    # perform post-load tasks
    _perform_onsessionloadcomplete_ops()

    # log load time and thumbs-up :)
    endtime = timer.get_time()
    success_emoji = ':OK_hand:' if endtime < 3.00 else ':thumbs_up:'
    mlogger.info('Load time: %s seconds %s', endtime, success_emoji)

    # if everything went well, self destruct
    try:
        timeout = user_config.startuplog_timeout
        if timeout > 0 and not logger.loggers_have_errors():
            if EXEC_PARAMS.first_load:
                # output_window is of type ScriptConsole
                output_window.SelfDestructTimer(timeout)
            else:
                # output_window is of type PyRevitOutputWindow
                output_window.self_destruct(timeout)
    except Exception as imp_err:
        mlogger.error('Error setting up self_destruct on output window | %s',
                      imp_err)

    _cleanup_output()
    return sessioninfo.get_session_uuid()


def _perform_onsessionreload_ops():
    pass


def _perform_onsessionreloadcomplete_ops():
    pass


def reload_pyrevit():
    _perform_onsessionreload_ops()
    mlogger.info('Reloading....')
    session_Id = load_session()
    _perform_onsessionreloadcomplete_ops()
    return session_Id

# -----------------------------------------------------------------------------
# Functions related to finding/executing
# pyrevit command or script in current session
# -----------------------------------------------------------------------------
class PyRevitExternalCommandType(object):
    """PyRevit external command type."""
    def __init__(self, extcmd_type, extcmd_availtype):
        self._extcmd_type = extcmd_type
        self._extcmd = extcmd_type()
        if extcmd_availtype:
            self._extcmd_availtype = extcmd_availtype
            self._extcmd_avail = extcmd_availtype()
        else:
            self._extcmd_availtype = None
            self._extcmd_avail = None

    @property
    def extcmd_type(self):
        return self._extcmd_type

    @property
    def typename(self):
        return self._extcmd_type.FullName

    @property
    def extcmd_availtype(self):
        return self._extcmd_availtype

    @property
    def avail_typename(self):
        return self._extcmd_availtype.FullName

    @property
    def script(self):
        return getattr(self._extcmd.ScriptData, 'ScriptPath', None)

    @property
    def config_script(self):
        return getattr(self._extcmd.ScriptData, 'ConfigScriptPath', None)

    @property
    def search_paths(self):
        value = getattr(self._extcmd.ScriptRuntimeConfigs, 'SearchPaths', [])
        return list(value)

    @property
    def arguments(self):
        value = getattr(self._extcmd.ScriptRuntimeConfigs, 'Arguments', [])
        return list(value)

    @property
    def engine_cfgs(self):
        return getattr(self._extcmd.ScriptRuntimeConfigs, 'EngineConfigs', '')

    @property
    def helpsource(self):
        return getattr(self._extcmd.ScriptData, 'HelpSource', None)

    @property
    def tooltip(self):
        return getattr(self._extcmd.ScriptData, 'Tooltip', None)

    @property
    def name(self):
        return getattr(self._extcmd.ScriptData, 'CommandName', None)

    @property
    def bundle(self):
        return getattr(self._extcmd.ScriptData, 'CommandBundle', None)

    @property
    def extension(self):
        return getattr(self._extcmd.ScriptData, 'CommandExtension', None)

    @property
    def unique_id(self):
        return getattr(self._extcmd.ScriptData, 'CommandUniqueId', None)

    def is_available(self, category_set, zerodoc=False):
        if self._extcmd_availtype:
            return self._extcmd_avail.IsCommandAvailable(HOST_APP.uiapp,
                                                         category_set)
        elif not zerodoc:
            return True

        return False


pyrevit_extcmdtype_cache = []


def find_all_commands(category_set=None, cache=True):
    global pyrevit_extcmdtype_cache    #pylint: disable=W0603
    if cache and pyrevit_extcmdtype_cache:    #pylint: disable=E0601
        pyrevit_extcmds = pyrevit_extcmdtype_cache
    else:
        pyrevit_extcmds = []
        for loaded_assm_name in sessioninfo.get_loaded_pyrevit_assemblies():
            loaded_assm = assmutils.find_loaded_asm(loaded_assm_name)
            if loaded_assm:
                all_exported_types = loaded_assm[0].GetTypes()

                for pyrvt_type in all_exported_types:
                    tname = pyrvt_type.FullName
                    availtname = pyrvt_type.Name \
                                 + runtime.CMD_AVAIL_NAME_POSTFIX
                    pyrvt_availtype = None

                    if not tname.endswith(runtime.CMD_AVAIL_NAME_POSTFIX)\
                            and runtime.RUNTIME_NAMESPACE not in tname:
                        for exported_type in all_exported_types:
                            if exported_type.Name == availtname:
                                pyrvt_availtype = exported_type

                        pyrevit_extcmds.append(
                            PyRevitExternalCommandType(pyrvt_type,
                                                       pyrvt_availtype)
                            )
        if cache:
            pyrevit_extcmdtype_cache = pyrevit_extcmds

    # now check commands in current context if requested
    if category_set:
        return [x for x in pyrevit_extcmds
                if x.is_available(category_set=category_set,
                                  zerodoc=HOST_APP.uidoc is None)]
    else:
        return pyrevit_extcmds


def find_all_available_commands(use_current_context=True, cache=True):
    if use_current_context:
        cset = revit.get_selection_category_set()
    else:
        cset = None

    return find_all_commands(category_set=cset, cache=cache)


def find_pyrevitcmd(pyrevitcmd_unique_id):
    """Find a pyRevit command.

    Searches the pyRevit-generated assemblies under current session for
    the command with the matching unique name (class name) and returns the
    command type. Notice that this returned value is a 'type' and should be
    instantiated before use.

    Examples:
        ```python
        cmd = find_pyrevitcmd('pyRevitCorepyRevitpyRevittoolsReload')
        command_instance = cmd()
        command_instance.Execute() # Provide commandData, message, elements
        ```

    Args:
        pyrevitcmd_unique_id (str): Unique name for the command

    Returns:
        (type):Type for the command with matching unique name
    """
    # go through assmebles loaded under current pyRevit session
    # and try to find the command
    mlogger.debug('Searching for pyrevit command: %s', pyrevitcmd_unique_id)
    for loaded_assm_name in sessioninfo.get_loaded_pyrevit_assemblies():
        mlogger.debug('Expecting assm: %s', loaded_assm_name)
        loaded_assm = assmutils.find_loaded_asm(loaded_assm_name)
        if loaded_assm:
            mlogger.debug('Found assm: %s', loaded_assm_name)
            for pyrvt_type in loaded_assm[0].GetTypes():
                mlogger.debug('Found Type: %s', pyrvt_type)
                if pyrvt_type.FullName == pyrevitcmd_unique_id:
                    mlogger.debug('Found pyRevit command in %s',
                                  loaded_assm_name)
                    return pyrvt_type
            mlogger.debug('Could not find pyRevit command.')
        else:
            mlogger.debug('Can not find assm: %s', loaded_assm_name)

    return None


def create_tmp_commanddata():
    tmp_cmd_data = \
        framework.FormatterServices.GetUninitializedObject(
            UI.ExternalCommandData
            )
    tmp_cmd_data.Application = HOST_APP.uiapp
    # tmp_cmd_data.IsReadOnly = False
    # tmp_cmd_data.View = None
    # tmp_cmd_data.JournalData = None
    return tmp_cmd_data


def execute_command_cls(extcmd_type, arguments=None,
                        config_mode=False, exec_from_ui=False):

    command_instance = extcmd_type()
    # pass the arguments to the instance
    if arguments:
        command_instance.ScriptRuntimeConfigs.Arguments = \
            framework.List[str](arguments)
    # this is a manual execution from python code and not by user
    command_instance.ExecConfigs.MimicExecFromUI = exec_from_ui
    # force using the config script
    command_instance.ExecConfigs.UseConfigScript = config_mode

    # Execute(
    # ExternalCommandData commandData,
    # string message,
    # ElementSet elements
    # )
    re = command_instance.Execute(create_tmp_commanddata(),
                                  '',
                                  DB.ElementSet())
    command_instance = None
    return re


def execute_command(pyrevitcmd_unique_id):
    """Executes a pyRevit command.

    Args:
        pyrevitcmd_unique_id (str): Unique/Class Name of the pyRevit command
    """
    cmd_class = find_pyrevitcmd(pyrevitcmd_unique_id)

    if not cmd_class:
        mlogger.error('Can not find command with unique name: %s',
                      pyrevitcmd_unique_id)
        return None
    else:
        execute_command_cls(cmd_class)


def execute_extension_startup_script(script_path, ext_name, sys_paths=None):
    """Executes a script using pyRevit script executor.

    Args:
        script_path (str): Address of the script file
        ext_name (str): Name of the extension
        sys_paths (list): additional search paths
    """
    core_syspaths = [MAIN_LIB_DIR, MISC_LIB_DIR]
    if sys_paths:
        sys_paths.extend(core_syspaths)
    else:
        sys_paths = core_syspaths

    script_data = runtime.types.ScriptData()
    script_data.ScriptPath = script_path
    script_data.ConfigScriptPath = None
    script_data.CommandUniqueId = ''
    script_data.CommandName = 'Starting {}'.format(ext_name)
    script_data.CommandBundle = ''
    script_data.CommandExtension = ext_name
    script_data.HelpSource = ''

    script_runtime_cfg = runtime.types.ScriptRuntimeConfigs()
    script_runtime_cfg.CommandData = create_tmp_commanddata()
    script_runtime_cfg.SelectedElements = None
    script_runtime_cfg.SearchPaths = framework.List[str](sys_paths or [])
    script_runtime_cfg.Arguments = framework.List[str]([])
    script_runtime_cfg.EngineConfigs = \
        runtime.create_ipyengine_configs(
            clean=True,
            full_frame=True,
            persistent=True,
        )
    script_runtime_cfg.RefreshEngine = False
    script_runtime_cfg.ConfigMode = False
    script_runtime_cfg.DebugMode = False
    script_runtime_cfg.ExecutedFromUI = False

    runtime.types.ScriptExecutor.ExecuteScript(
        script_data,
        script_runtime_cfg
    )



class IEasingFunction:
 """ Defines the basic functionality of an easing function. """
 def Ease(self,normalizedTime):
  """
  Ease(self: IEasingFunction,normalizedTime: float) -> float

  

   Transforms normalized time to control the pace of an animation.

  

   normalizedTime: Normalized time (progress) of the animation.

   Returns: The transformed progress.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass



# encoding: utf-8
# module System.Windows.Interop calls itself Interop
# from PresentationCore,Version=4.0.0.0,Culture=neutral,PublicKeyToken=31bf3856ad364e35
# by generator 1.145
""" NamespaceTracker represent a CLS namespace. """
# no imports

# no functions
# classes

class CursorInteropHelper(object):
 """ Provides a static helper class for WPF/Win32 interoperation with one method,which is used to obtain a Windows Presentation Foundation (WPF)�System.Windows.Input.Cursor object based on a provided Win32 cursor handle. """
 @staticmethod
 def Create(cursorHandle):
  """
  Create(cursorHandle: SafeHandle) -> Cursor

  

   Returns a Windows Presentation Foundation (WPF)�System.Windows.Input.Cursor object based on a 

    provided Win32 cursor handle.

  

  

   cursorHandle: Cursor reference to use for interoperation.

   Returns: The Windows Presentation Foundation (WPF)�cursor object based on the provided Win32 cursor 

    handle.
  """
  pass
 __all__=[
  'Create',
 ]


class D3DImage(ImageSource,ISealable,IAnimatable,IResource,IFormattable,IAppDomainShutdownListener):
 """
 An System.Windows.Media.ImageSource that displays a user-created Direct3D surface.

 

 D3DImage()

 D3DImage(dpiX: float,dpiY: float)
 """
 def AddDirtyRect(self,dirtyRect):
  """
  AddDirtyRect(self: D3DImage,dirtyRect: Int32Rect)

   Specifies the area of the back buffer that changed.

  

   dirtyRect: An System.Windows.Int32Rect that represents the area that changed.
  """
  pass
 def Clone(self):
  """
  Clone(self: D3DImage) -> D3DImage

  

   Creates a modifiable clone of this System.Windows.Interop.D3DImage object,making deep copies of 

    this object's values. When copying dependency properties,this method copies resource references 

    and data bindings (which may no longer resolve),but not animations or their current values.

  

   Returns: A modifiable clone of the current object. The cloned object's System.Windows.Freezable.IsFrozen 

    property will be false even if the source's System.Windows.Freezable.IsFrozen property was true.
  """
  pass
 def CloneCore(self,*args):
  """
  CloneCore(self: D3DImage,sourceFreezable: Freezable)

   sourceFreezable: The object to clone.
  """
  pass
 def CloneCurrentValue(self):
  """
  CloneCurrentValue(self: D3DImage) -> D3DImage

  

   Creates a modifiable clone of this System.Windows.Interop.D3DImage object,making deep copies of 

    this object's current values. Resource references,data bindings,and animations are not copied,

    but their current values are copied.

  

   Returns: A modifiable clone of the current object. The cloned object's System.Windows.Freezable.IsFrozen 

    property will be false even if the source's System.Windows.Freezable.IsFrozen property was true.
  """
  pass
 def CloneCurrentValueCore(self,*args):
  """
  CloneCurrentValueCore(self: D3DImage,sourceFreezable: Freezable)

   sourceFreezable: The System.Windows.Freezable to be cloned.
  """
  pass
 def CopyBackBuffer(self,*args):
  """
  CopyBackBuffer(self: D3DImage) -> BitmapSource

  

   Creates a software copy of the System.Windows.Interop.D3DImage.

   Returns: A System.Windows.Media.Imaging.BitmapSource that is a software copy of the current state of the 

    back buffer; otherwise,null if the back buffer cannot be read.
  """
  pass
 def CreateInstance(self,*args):
  """
  CreateInstance(self: Freezable) -> Freezable

  

   Initializes a new instance of the System.Windows.Freezable class.

   Returns: The new instance.
  """
  pass
 def CreateInstanceCore(self,*args):
  """
  CreateInstanceCore(self: D3DImage) -> Freezable

  

   When implemented in a derived class,creates a new instance of the 

    System.Windows.Interop.D3DImage derived class.

  

   Returns: The new instance.
  """
  pass
 def FreezeCore(self,*args):
  """
  FreezeCore(self: D3DImage,isChecking: bool) -> bool

  

   Makes the System.Windows.Interop.D3DImage unmodifiable or determines whether it can be made 

    unmodifiable.

  

  

   isChecking: Has no effect.

   Returns: false in all cases.
  """
  pass
 def GetAsFrozenCore(self,*args):
  """
  GetAsFrozenCore(self: D3DImage,sourceFreezable: Freezable)

   sourceFreezable: The instance to copy.
  """
  pass
 def GetCurrentValueAsFrozenCore(self,*args):
  """
  GetCurrentValueAsFrozenCore(self: D3DImage,sourceFreezable: Freezable)

   sourceFreezable: The System.Windows.Freezable to copy and freeze.
  """
  pass
 def Lock(self):
  """
  Lock(self: D3DImage)

   Locks the System.Windows.Interop.D3DImage and enables operations on the back buffer.
  """
  pass
 def OnChanged(self,*args):
  """
  OnChanged(self: Freezable)

   Called when the current System.Windows.Freezable object is modified.
  """
  pass
 def OnFreezablePropertyChanged(self,*args):
  """
  OnFreezablePropertyChanged(self: Freezable,oldValue: DependencyObject,newValue: DependencyObject,property: DependencyProperty)

   This member supports the Windows Presentation Foundation (WPF) infrastructure and is not 

    intended to be used directly from your code.

  

  

   oldValue: The previous value of the data member.

   newValue: The current value of the data member.

   property: The property that changed.

  OnFreezablePropertyChanged(self: Freezable,oldValue: DependencyObject,newValue: DependencyObject)

   Ensures that appropriate context pointers are established for a 

    System.Windows.DependencyObjectType data member that has just been set.

  

  

   oldValue: The previous value of the data member.

   newValue: The current value of the data member.
  """
  pass
 def OnPropertyChanged(self,*args):
  """
  OnPropertyChanged(self: Freezable,e: DependencyPropertyChangedEventArgs)

   Overrides the System.Windows.DependencyObject implementation of 

    System.Windows.DependencyObject.OnPropertyChanged(System.Windows.DependencyPropertyChangedEventAr

    gs) to also invoke any System.Windows.Freezable.Changed handlers in response to a changing 

    dependency property of type System.Windows.Freezable.

  

  

   e: Event data that contains information about which property changed,and its old and new values.
  """
  pass
 def ReadPreamble(self,*args):
  """
  ReadPreamble(self: Freezable)

   Ensures that the System.Windows.Freezable is being accessed from a valid thread. Inheritors of 

    System.Windows.Freezable must call this method at the beginning of any API that reads data 

    members that are not dependency properties.
  """
  pass
 def SetBackBuffer(self,backBufferType,backBuffer,enableSoftwareFallback=None):
  """
  SetBackBuffer(self: D3DImage,backBufferType: D3DResourceType,backBuffer: IntPtr,enableSoftwareFallback: bool)SetBackBuffer(self: D3DImage,backBufferType: D3DResourceType,backBuffer: IntPtr)

   Assigns a Direct3D surface as the source of the back buffer.

  

   backBufferType: The type of Direct3D surface. Must be a valid System.Windows.Interop.D3DResourceType.

   backBuffer: The Direct3D surface to assign as the back buffer.
  """
  pass
 def ShouldSerializeProperty(self,*args):
  """
  ShouldSerializeProperty(self: DependencyObject,dp: DependencyProperty) -> bool

  

   Returns a value that indicates whether serialization processes should serialize the value for 

    the provided dependency property.

  

  

   dp: The identifier for the dependency property that should be serialized.

   Returns: true if the dependency property that is supplied should be value-serialized; otherwise,false.
  """
  pass
 def TryLock(self,timeout):
  """
  TryLock(self: D3DImage,timeout: Duration) -> bool

  

   Attempts to lock the System.Windows.Interop.D3DImage and waits for the specified duration.

  

   timeout: The duration to wait for the lock to be acquired.

   Returns: true if the lock was acquired; otherwise,false.
  """
  pass
 def Unlock(self):
  """
  Unlock(self: D3DImage)

   Decrements the lock count for the System.Windows.Interop.D3DImage.
  """
  pass
 def WritePostscript(self,*args):
  """
  WritePostscript(self: Freezable)

   Raises the System.Windows.Freezable.Changed event for the System.Windows.Freezable and invokes 

    its System.Windows.Freezable.OnChanged method. Classes that derive from System.Windows.Freezable 

    should call this method at the end of any API that modifies class members that are not stored as 

    dependency properties.
  """
  pass
 def WritePreamble(self,*args):
  """
  WritePreamble(self: Freezable)

   Verifies that the System.Windows.Freezable is not frozen and that it is being accessed from a 

    valid threading context. System.Windows.Freezable inheritors should call this method at the 

    beginning of any API that writes to data members that are not dependency properties.
  """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,dpiX=None,dpiY=None):
  """
  __new__(cls: type)

  __new__(cls: type,dpiX: float,dpiY: float)
  """
  pass
 def __str__(self,*args):
  pass
 Height=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the height of the System.Windows.Interop.D3DImage.



Get: Height(self: D3DImage) -> float



"""

 IsFrontBufferAvailable=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether a front buffer exists.



Get: IsFrontBufferAvailable(self: D3DImage) -> bool



"""

 Metadata=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the metadata associated with the image source.



Get: Metadata(self: D3DImage) -> ImageMetadata



"""

 PixelHeight=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the height of the System.Windows.Interop.D3DImage,in pixels.



Get: PixelHeight(self: D3DImage) -> int



"""

 PixelWidth=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the width of the System.Windows.Interop.D3DImage,in pixels.



Get: PixelWidth(self: D3DImage) -> int



"""

 Width=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the width of the System.Windows.Interop.D3DImage.



Get: Width(self: D3DImage) -> float



"""


 IsFrontBufferAvailableChanged=None
 IsFrontBufferAvailableProperty=None


class D3DResourceType(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies the Direct3D surface types that are compatible with the System.Windows.Interop.D3DImage class.

 

 enum D3DResourceType,values: IDirect3DSurface9 (0)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 IDirect3DSurface9=None
 value__=None


class IWin32Window:
 """ Defines the contract for Win32 window handles. """
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 Handle=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the window handle.



Get: Handle(self: IWin32Window) -> IntPtr



"""



class HwndSource(PresentationSource,IDisposable,IWin32Window,IKeyboardInputSink):
 """
 Presents Windows Presentation Foundation (WPF) content in a Win32 window.

 

 HwndSource(classStyle: int,style: int,exStyle: int,x: int,y: int,name: str,parent: IntPtr)

 HwndSource(classStyle: int,style: int,exStyle: int,x: int,y: int,width: int,height: int,name: str,parent: IntPtr,adjustSizingForNonClientArea: bool)

 HwndSource(classStyle: int,style: int,exStyle: int,x: int,y: int,width: int,height: int,name: str,parent: IntPtr)

 HwndSource(parameters: HwndSourceParameters)
 """
 def AddHook(self,hook):
  """
  AddHook(self: HwndSource,hook: HwndSourceHook)

   Adds an event handler that receives all window messages.

  

   hook: The handler implementation (based on the System.Windows.Interop.HwndSourceHook delegate) that 

    receives the window messages.
  """
  pass
 def AddSource(self,*args):
  """
  AddSource(self: PresentationSource)

   Adds a System.Windows.PresentationSource derived class instance to the list of known 

    presentation sources.
  """
  pass
 def ClearContentRenderedListeners(self,*args):
  """
  ClearContentRenderedListeners(self: PresentationSource)

   Sets the list of listeners for the System.Windows.PresentationSource.ContentRendered event to 

    null.
  """
  pass
 def CreateHandleRef(self):
  """
  CreateHandleRef(self: HwndSource) -> HandleRef

  

   Gets the window handle for the System.Windows.Interop.HwndSource. The window handle is packaged 

    as part of a System.Runtime.InteropServices.HandleRef structure.

  

   Returns: A structure that contains the window handle for this System.Windows.Interop.HwndSource.
  """
  pass
 def Dispose(self):
  """
  Dispose(self: HwndSource)

   Releases all managed resources that are used by the System.Windows.Interop.HwndSource,and 

    raises the System.Windows.Interop.HwndSource.Disposed event.
  """
  pass
 @staticmethod
 def FromHwnd(hwnd):
  """
  FromHwnd(hwnd: IntPtr) -> HwndSource

  

   Returns the System.Windows.Interop.HwndSource object of the specified window.

  

   hwnd: The provided window handle.

   Returns: The System.Windows.Interop.HwndSource object for the window that is specified by the hwnd window 

    handle.
  """
  pass
 def GetCompositionTargetCore(self,*args):
  """
  GetCompositionTargetCore(self: HwndSource) -> CompositionTarget

  

   Gets the visual target of the window.

   Returns: Returns the visual target of the window.
  """
  pass
 def HasFocusWithinCore(self,*args):
  """
  HasFocusWithinCore(self: HwndSource) -> bool

  

   Gets a value that indicates whether the sink or one of its contained components has focus.

   Returns: true if the sink or one of its contained components has focus; otherwise,false.
  """
  pass
 def OnDpiChanged(self,*args):
  """ OnDpiChanged(self: HwndSource,e: HwndDpiChangedEventArgs) """
  pass
 def OnMnemonicCore(self,*args):
  """
  OnMnemonicCore(self: HwndSource,msg: MSG,modifiers: ModifierKeys) -> (bool,MSG)

  

   Called when one of the mnemonics (access keys) for this sink is invoked.

  

   msg: The message for the mnemonic and associated data.

   modifiers: Modifier keys.

   Returns: true if the message was handled; otherwise,false.
  """
  pass
 def RegisterKeyboardInputSinkCore(self,*args):
  """
  RegisterKeyboardInputSinkCore(self: HwndSource,sink: IKeyboardInputSink) -> IKeyboardInputSite

  

   Registers the System.Windows.Interop.IKeyboardInputSink interface of a contained component.

  

   sink: The System.Windows.Interop.IKeyboardInputSink sink of the contained component.

   Returns: The System.Windows.Interop.IKeyboardInputSite site of the contained component.
  """
  pass
 def RemoveHook(self,hook):
  """
  RemoveHook(self: HwndSource,hook: HwndSourceHook)

   Removes the event handlers that were added by 

    System.Windows.Interop.HwndSource.AddHook(System.Windows.Interop.HwndSourceHook).

  

  

   hook: The event handler to remove.
  """
  pass
 def RemoveSource(self,*args):
  """
  RemoveSource(self: PresentationSource)

   Removes a System.Windows.PresentationSource derived class instance from the list of known 

    presentation sources.
  """
  pass
 def RootChanged(self,*args):
  """
  RootChanged(self: PresentationSource,oldRoot: Visual,newRoot: Visual)

   Provides notification that the root System.Windows.Media.Visual has changed.

  

   oldRoot: The old root System.Windows.Media.Visual.

   newRoot: The new root System.Windows.Media.Visual.
  """
  pass
 def TabIntoCore(self,*args):
  """
  TabIntoCore(self: HwndSource,request: TraversalRequest) -> bool

  

   Sets focus on either the first tab stop or the last tab stop of the sink.

  

   request: Specifies whether focus should be set to the first or the last tab stop.

   Returns: true if the focus has been set as requested; false,if there are no tab stops.
  """
  pass
 def TranslateAcceleratorCore(self,*args):
  """
  TranslateAcceleratorCore(self: HwndSource,msg: MSG,modifiers: ModifierKeys) -> (bool,MSG)

  

   Processes keyboard input at the key-down message level.

  

   msg: The message and associated data. Do not modify this structure. It is passed by reference for 

    performance reasons only.

  

   modifiers: Modifier keys.

   Returns: true if the message was handled by the method implementation; otherwise,false.
  """
  pass
 def TranslateCharCore(self,*args):
  """
  TranslateCharCore(self: HwndSource,msg: MSG,modifiers: ModifierKeys) -> (bool,MSG)

  

   Processes WM_CHAR,WM_SYSCHAR,WM_DEADCHAR,and WM_SYSDEADCHAR input messages before the 

    System.Windows.Interop.IKeyboardInputSink.OnMnemonic(System.Windows.Interop.MSG@,System.Windows.I

    nput.ModifierKeys) method is called.

  

  

   msg: The message and associated data. Do not modify this structure. It is passed by reference for 

    performance reasons only.

  

   modifiers: Modifier keys.

   Returns: true if the message was processed and 

    System.Windows.Interop.IKeyboardInputSink.OnMnemonic(System.Windows.Interop.MSG@,System.Windows.I

    nput.ModifierKeys) should not be called; otherwise,false.
  """
  pass
 def __enter__(self,*args):
  """
  __enter__(self: IDisposable) -> object

  

   Provides the implementation of __enter__ for objects which implement IDisposable.
  """
  pass
 def __exit__(self,*args):
  """
  __exit__(self: IDisposable,exc_type: object,exc_value: object,exc_back: object)

   Provides the implementation of __exit__ for objects which implement IDisposable.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,*__args):
  """
  __new__(cls: type,classStyle: int,style: int,exStyle: int,x: int,y: int,name: str,parent: IntPtr)

  __new__(cls: type,classStyle: int,style: int,exStyle: int,x: int,y: int,width: int,height: int,name: str,parent: IntPtr,adjustSizingForNonClientArea: bool)

  __new__(cls: type,classStyle: int,style: int,exStyle: int,x: int,y: int,width: int,height: int,name: str,parent: IntPtr)

  __new__(cls: type,parameters: HwndSourceParameters)
  """
  pass
 AcquireHwndFocusInMenuMode=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the value that determines whether to acquire Win32 focus for the WPF containing window for this System.Windows.Interop.HwndSource.



Get: AcquireHwndFocusInMenuMode(self: HwndSource) -> bool



"""

 ChildKeyboardInputSinks=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a sequence of registered input sinks.



Get: ChildKeyboardInputSinks(self: HwndSource) -> IEnumerable[IKeyboardInputSink]



"""

 CompositionTarget=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the visual manager for the hosted window.



Get: CompositionTarget(self: HwndSource) -> HwndTarget



"""

 Handle=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the window handle for this System.Windows.Interop.HwndSource.



Get: Handle(self: HwndSource) -> IntPtr



"""

 IsDisposed=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether System.Windows.Interop.HwndSource.Dispose has been called on this System.Windows.Interop.HwndSource.



Get: IsDisposed(self: HwndSource) -> bool



"""

 KeyboardInputSiteCore=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a reference to the component's container's System.Windows.Interop.IKeyboardInputSite interface.



"""

 RestoreFocusMode=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the System.Windows.Input.RestoreFocusMode for the window.



Get: RestoreFocusMode(self: HwndSource) -> RestoreFocusMode



"""

 RootVisual=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the System.Windows.Media.CompositionTarget.RootVisual of the window.



Get: RootVisual(self: HwndSource) -> Visual



Set: RootVisual(self: HwndSource)=value

"""

 SizeToContent=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Get or sets whether and how the window is sized to its content.



Get: SizeToContent(self: HwndSource) -> SizeToContent



Set: SizeToContent(self: HwndSource)=value

"""

 UsesPerPixelOpacity=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that declares whether the per-pixel opacity of the source window content is respected.



Get: UsesPerPixelOpacity(self: HwndSource) -> bool



"""


 AutoResized=None
 DefaultAcquireHwndFocusInMenuMode=True
 Disposed=None
 DpiChanged=None
 SizeToContentChanged=None


class HwndSourceHook(MulticastDelegate,ICloneable,ISerializable):
 """
 Represents the method that handles Win32 window messages.

 

 HwndSourceHook(object: object,method: IntPtr)
 """
 def BeginInvoke(self,hwnd,msg,wParam,lParam,handled,callback,object):
  """ BeginInvoke(self: HwndSourceHook,hwnd: IntPtr,msg: int,wParam: IntPtr,lParam: IntPtr,handled: bool,callback: AsyncCallback,object: object) -> (IAsyncResult,bool) """
  pass
 def CombineImpl(self,*args):
  """
  CombineImpl(self: MulticastDelegate,follow: Delegate) -> Delegate

  

   Combines this System.Delegate with the specified System.Delegate to form a new delegate.

  

   follow: The delegate to combine with this delegate.

   Returns: A delegate that is the new root of the System.MulticastDelegate invocation list.
  """
  pass
 def DynamicInvokeImpl(self,*args):
  """
  DynamicInvokeImpl(self: Delegate,args: Array[object]) -> object

  

   Dynamically invokes (late-bound) the method represented by the current delegate.

  

   args: An array of objects that are the arguments to pass to the method represented by the current 

    delegate.-or- null,if the method represented by the current delegate does not require 

    arguments.

  

   Returns: The object returned by the method represented by the delegate.
  """
  pass
 def EndInvoke(self,handled,result):
  """ EndInvoke(self: HwndSourceHook,handled: bool,result: IAsyncResult) -> (IntPtr,bool) """
  pass
 def GetMethodImpl(self,*args):
  """
  GetMethodImpl(self: MulticastDelegate) -> MethodInfo

  

   Returns a static method represented by the current System.MulticastDelegate.

   Returns: A static method represented by the current System.MulticastDelegate.
  """
  pass
 def Invoke(self,hwnd,msg,wParam,lParam,handled):
  """ Invoke(self: HwndSourceHook,hwnd: IntPtr,msg: int,wParam: IntPtr,lParam: IntPtr,handled: bool) -> (IntPtr,bool) """
  pass
 def RemoveImpl(self,*args):
  """
  RemoveImpl(self: MulticastDelegate,value: Delegate) -> Delegate

  

   Removes an element from the invocation list of this System.MulticastDelegate that is equal to 

    the specified delegate.

  

  

   value: The delegate to search for in the invocation list.

   Returns: If value is found in the invocation list for this instance,then a new System.Delegate without 

    value in its invocation list; otherwise,this instance with its original invocation list.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,object,method):
  """ __new__(cls: type,object: object,method: IntPtr) """
  pass
 def __reduce_ex__(self,*args):
  pass

class HwndSourceParameters(object):
 """
 Contains the parameters that are used to create an System.Windows.Interop.HwndSource object using the System.Windows.Interop.HwndSource.#ctor(System.Windows.Interop.HwndSourceParameters) constructor.

 

 HwndSourceParameters(name: str)

 HwndSourceParameters(name: str,width: int,height: int)
 """
 def Equals(self,obj):
  """
  Equals(self: HwndSourceParameters,obj: HwndSourceParameters) -> bool

  

   Determines whether this structure is equal to a specified 

    System.Windows.Interop.HwndSourceParameters structure.

  

  

   obj: The structure to be tested for equality.

   Returns: true if the structures are equal; otherwise,false.

  Equals(self: HwndSourceParameters,obj: object) -> bool

  

   Determines whether this structure is equal to a specified object.

  

   obj: The objects to be tested for equality.

   Returns: true if the comparison is equal; otherwise,false.
  """
  pass
 def GetHashCode(self):
  """
  GetHashCode(self: HwndSourceParameters) -> int

  

   Returns the hash code for this System.Windows.Interop.HwndSourceParameters instance.

   Returns: A 32-bit signed integer hash code.
  """
  pass
 def SetPosition(self,x,y):
  """
  SetPosition(self: HwndSourceParameters,x: int,y: int)

   Sets the values that are used for the screen position of the window for the 

    System.Windows.Interop.HwndSource.

  

  

   x: The position of the left edge of the window.

   y: The position of the upper edge of the window.
  """
  pass
 def SetSize(self,width,height):
  """
  SetSize(self: HwndSourceParameters,width: int,height: int)

   Sets the values that are used for the window size of the System.Windows.Interop.HwndSource.

  

   width: The width of the window,in device pixels.

   height: The height of the window,in device pixels.
  """
  pass
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==y """
  pass
 @staticmethod
 def __new__(self,name,width=None,height=None):
  """
  __new__(cls: type,name: str)

  __new__(cls: type,name: str,width: int,height: int)

  __new__[HwndSourceParameters]() -> HwndSourceParameters
  """
  pass
 def __ne__(self,*args):
  pass
 AcquireHwndFocusInMenuMode=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the value that determines whether to acquire Win32 focus for the WPF containing window when an System.Windows.Interop.HwndSource is created.



Get: AcquireHwndFocusInMenuMode(self: HwndSourceParameters) -> bool



Set: AcquireHwndFocusInMenuMode(self: HwndSourceParameters)=value

"""

 AdjustSizingForNonClientArea=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value that indicates whether to include the nonclient area for sizing.



Get: AdjustSizingForNonClientArea(self: HwndSourceParameters) -> bool



Set: AdjustSizingForNonClientArea(self: HwndSourceParameters)=value

"""

 ExtendedWindowStyle=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the extended Microsoft Windows styles for the window.



Get: ExtendedWindowStyle(self: HwndSourceParameters) -> int



Set: ExtendedWindowStyle(self: HwndSourceParameters)=value

"""

 HasAssignedSize=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether a size was assigned.



Get: HasAssignedSize(self: HwndSourceParameters) -> bool



"""

 Height=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value that indicates the height of the window.



Get: Height(self: HwndSourceParameters) -> int



Set: Height(self: HwndSourceParameters)=value

"""

 HwndSourceHook=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the message hook for the window.



Get: HwndSourceHook(self: HwndSourceParameters) -> HwndSourceHook



Set: HwndSourceHook(self: HwndSourceParameters)=value

"""

 ParentWindow=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the window handle (HWND) of the parent for the created window.



Get: ParentWindow(self: HwndSourceParameters) -> IntPtr



Set: ParentWindow(self: HwndSourceParameters)=value

"""

 PositionX=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the left-edge position of the window.



Get: PositionX(self: HwndSourceParameters) -> int



Set: PositionX(self: HwndSourceParameters)=value

"""

 PositionY=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the upper-edge position of the window.



Get: PositionY(self: HwndSourceParameters) -> int



Set: PositionY(self: HwndSourceParameters)=value

"""

 RestoreFocusMode=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets how WPF handles restoring focus to the window.



Get: RestoreFocusMode(self: HwndSourceParameters) -> RestoreFocusMode



Set: RestoreFocusMode(self: HwndSourceParameters)=value

"""

 TreatAncestorsAsNonClientArea=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Get: TreatAncestorsAsNonClientArea(self: HwndSourceParameters) -> bool



Set: TreatAncestorsAsNonClientArea(self: HwndSourceParameters)=value

"""

 TreatAsInputRoot=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Get: TreatAsInputRoot(self: HwndSourceParameters) -> bool



Set: TreatAsInputRoot(self: HwndSourceParameters)=value

"""

 UsesPerPixelOpacity=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that declares whether the per-pixel opacity of the source window content is respected.



Get: UsesPerPixelOpacity(self: HwndSourceParameters) -> bool



Set: UsesPerPixelOpacity(self: HwndSourceParameters)=value

"""

 UsesPerPixelTransparency=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Get: UsesPerPixelTransparency(self: HwndSourceParameters) -> bool



Set: UsesPerPixelTransparency(self: HwndSourceParameters)=value

"""

 Width=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value that indicates the width of the window.



Get: Width(self: HwndSourceParameters) -> int



Set: Width(self: HwndSourceParameters)=value

"""

 WindowClassStyle=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the Microsoft Windows class style for the window.



Get: WindowClassStyle(self: HwndSourceParameters) -> int



Set: WindowClassStyle(self: HwndSourceParameters)=value

"""

 WindowName=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the name of the window.



Get: WindowName(self: HwndSourceParameters) -> str



Set: WindowName(self: HwndSourceParameters)=value

"""

 WindowStyle=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the style for the window.



Get: WindowStyle(self: HwndSourceParameters) -> int



Set: WindowStyle(self: HwndSourceParameters)=value

"""



class HwndTarget(CompositionTarget,IDisposable,ICompositionTarget):
 """
 Represents a binding to a window handle that supports visual composition.

 

 HwndTarget(hwnd: IntPtr)
 """
 def Dispose(self):
  """
  Dispose(self: HwndTarget)

   Releases all resources used by the System.Windows.Interop.HwndTarget.
  """
  pass
 def __enter__(self,*args):
  """
  __enter__(self: IDisposable) -> object

  

   Provides the implementation of __enter__ for objects which implement IDisposable.
  """
  pass
 def __exit__(self,*args):
  """
  __exit__(self: IDisposable,exc_type: object,exc_value: object,exc_back: object)

   Provides the implementation of __exit__ for objects which implement IDisposable.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,hwnd):
  """ __new__(cls: type,hwnd: IntPtr) """
  pass
 BackgroundColor=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the background color of the window referenced by this System.Windows.Interop.HwndTarget.



Get: BackgroundColor(self: HwndTarget) -> Color



Set: BackgroundColor(self: HwndTarget)=value

"""

 RenderMode=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the rendering mode for the window referenced by this System.Windows.Interop.HwndTarget.



Get: RenderMode(self: HwndTarget) -> RenderMode



Set: RenderMode(self: HwndTarget)=value

"""

 RootVisual=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the root visual object of the page that is hosted by the window.



Set: RootVisual(self: HwndTarget)=value

"""

 TransformFromDevice=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a matrix that transforms the coordinates of the device that is associated with the rendering destination of this target.



Get: TransformFromDevice(self: HwndTarget) -> Matrix



"""

 TransformToDevice=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a matrix that transforms the coordinates of this target to the device that is associated with the rendering destination.



Get: TransformToDevice(self: HwndTarget) -> Matrix



"""

 UsesPerPixelOpacity=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that declares whether the per-pixel opacity value of the source window content is used for rendering.



Get: UsesPerPixelOpacity(self: HwndTarget) -> bool



"""



class Imaging(object):
 """ Provides managed to unmanaged interoperation support for creating image objects. """
 @staticmethod
 def CreateBitmapSourceFromHBitmap(bitmap,palette,sourceRect,sizeOptions):
  """
  CreateBitmapSourceFromHBitmap(bitmap: IntPtr,palette: IntPtr,sourceRect: Int32Rect,sizeOptions: BitmapSizeOptions) -> BitmapSource

  

   Returns a managed System.Windows.Media.Imaging.BitmapSource,based on the provided pointer to an 

    unmanaged bitmap and palette information.

  

  

   bitmap: A pointer to the unmanaged bitmap.

   palette: A pointer to the bitmap's palette map.

   sourceRect: The size of the source image.

   sizeOptions: A value of the enumeration that specifies how to handle conversions.

   Returns: The created System.Windows.Media.Imaging.BitmapSource.
  """
  pass
 @staticmethod
 def CreateBitmapSourceFromHIcon(icon,sourceRect,sizeOptions):
  """
  CreateBitmapSourceFromHIcon(icon: IntPtr,sourceRect: Int32Rect,sizeOptions: BitmapSizeOptions) -> BitmapSource

  

   Returns a managed System.Windows.Media.Imaging.BitmapSource,based on the provided pointer to an 

    unmanaged icon image.

  

  

   icon: A pointer to the unmanaged icon source.

   sourceRect: The size of the source image.

   sizeOptions: A value of the enumeration that specifies how to handle conversions.

   Returns: The created System.Windows.Media.Imaging.BitmapSource.
  """
  pass
 @staticmethod
 def CreateBitmapSourceFromMemorySection(section,pixelWidth,pixelHeight,format,stride,offset):
  """
  CreateBitmapSourceFromMemorySection(section: IntPtr,pixelWidth: int,pixelHeight: int,format: PixelFormat,stride: int,offset: int) -> BitmapSource

  

   Returns a managed System.Windows.Media.Imaging.BitmapSource,based on the provided unmanaged 

    memory location.

  

  

   section: A pointer to a memory section.

   pixelWidth: An integer that specifies the width,in pixels,of the bitmap.

   pixelHeight: An integer that specifies the height,in pixels,of the bitmap.

   format: A value of the enumeration.

   stride: The stride of the bitmap.

   offset: The byte offset into the memory stream where the image starts.

   Returns: The created System.Windows.Media.Imaging.BitmapSource.
  """
  pass
 __all__=[
  'CreateBitmapSourceFromHBitmap',
  'CreateBitmapSourceFromHIcon',
  'CreateBitmapSourceFromMemorySection',
 ]


class InteropBitmap(BitmapSource,ISealable,IAnimatable,IResource,IFormattable):
 """ System.Windows.Interop.InteropBitmap enables developers to improve rendering performance of non-WPF�UIs that are hosted by WPF in interoperability scenarios. """
 def CheckIfSiteOfOrigin(self,*args):
  """
  CheckIfSiteOfOrigin(self: BitmapSource)

   Checks whether the bitmap source content is from a known site of origin. This method is used to 

    make sure that pixel copying operations are safe.
  """
  pass
 def CloneCore(self,*args):
  """ CloneCore(self: InteropBitmap,sourceFreezable: Freezable) """
  pass
 def CloneCurrentValueCore(self,*args):
  """ CloneCurrentValueCore(self: InteropBitmap,sourceFreezable: Freezable) """
  pass
 def CreateInstance(self,*args):
  """
  CreateInstance(self: Freezable) -> Freezable

  

   Initializes a new instance of the System.Windows.Freezable class.

   Returns: The new instance.
  """
  pass
 def CreateInstanceCore(self,*args):
  """ CreateInstanceCore(self: InteropBitmap) -> Freezable """
  pass
 def FreezeCore(self,*args):
  """
  FreezeCore(self: BitmapSource,isChecking: bool) -> bool

  

   Makes an instance of System.Windows.Media.Imaging.BitmapSource or a derived class immutable.

  

   isChecking: true if this instance should actually freeze itself when this method is called; otherwise,false.

   Returns: If isChecking is true,this method returns true if this 

    System.Windows.Media.Animation.Animatable can be made unmodifiable,or false if it cannot be 

    made unmodifiable. If isChecking is false,this method returns true if the if this 

    System.Windows.Media.Animation.Animatable is now unmodifiable,or false if it cannot be made 

    unmodifiable,with the side effect of having begun to change the frozen status of this object.
  """
  pass
 def GetAsFrozenCore(self,*args):
  """ GetAsFrozenCore(self: InteropBitmap,sourceFreezable: Freezable) """
  pass
 def GetCurrentValueAsFrozenCore(self,*args):
  """ GetCurrentValueAsFrozenCore(self: InteropBitmap,sourceFreezable: Freezable) """
  pass
 def Invalidate(self,dirtyRect=None):
  """
  Invalidate(self: InteropBitmap,dirtyRect: Nullable[Int32Rect])Invalidate(self: InteropBitmap)

   Forces the hosted non-WPF�UI to be rendered.
  """
  pass
 def OnChanged(self,*args):
  """
  OnChanged(self: Freezable)

   Called when the current System.Windows.Freezable object is modified.
  """
  pass
 def OnFreezablePropertyChanged(self,*args):
  """
  OnFreezablePropertyChanged(self: Freezable,oldValue: DependencyObject,newValue: DependencyObject,property: DependencyProperty)

   This member supports the Windows Presentation Foundation (WPF) infrastructure and is not 

    intended to be used directly from your code.

  

  

   oldValue: The previous value of the data member.

   newValue: The current value of the data member.

   property: The property that changed.

  OnFreezablePropertyChanged(self: Freezable,oldValue: DependencyObject,newValue: DependencyObject)

   Ensures that appropriate context pointers are established for a 

    System.Windows.DependencyObjectType data member that has just been set.

  

  

   oldValue: The previous value of the data member.

   newValue: The current value of the data member.
  """
  pass
 def OnPropertyChanged(self,*args):
  """
  OnPropertyChanged(self: Freezable,e: DependencyPropertyChangedEventArgs)

   Overrides the System.Windows.DependencyObject implementation of 

    System.Windows.DependencyObject.OnPropertyChanged(System.Windows.DependencyPropertyChangedEventAr

    gs) to also invoke any System.Windows.Freezable.Changed handlers in response to a changing 

    dependency property of type System.Windows.Freezable.

  

  

   e: Event data that contains information about which property changed,and its old and new values.
  """
  pass
 def ReadPreamble(self,*args):
  """
  ReadPreamble(self: Freezable)

   Ensures that the System.Windows.Freezable is being accessed from a valid thread. Inheritors of 

    System.Windows.Freezable must call this method at the beginning of any API that reads data 

    members that are not dependency properties.
  """
  pass
 def ShouldSerializeProperty(self,*args):
  """
  ShouldSerializeProperty(self: DependencyObject,dp: DependencyProperty) -> bool

  

   Returns a value that indicates whether serialization processes should serialize the value for 

    the provided dependency property.

  

  

   dp: The identifier for the dependency property that should be serialized.

   Returns: true if the dependency property that is supplied should be value-serialized; otherwise,false.
  """
  pass
 def WritePostscript(self,*args):
  """
  WritePostscript(self: Freezable)

   Raises the System.Windows.Freezable.Changed event for the System.Windows.Freezable and invokes 

    its System.Windows.Freezable.OnChanged method. Classes that derive from System.Windows.Freezable 

    should call this method at the end of any API that modifies class members that are not stored as 

    dependency properties.
  """
  pass
 def WritePreamble(self,*args):
  """
  WritePreamble(self: Freezable)

   Verifies that the System.Windows.Freezable is not frozen and that it is being accessed from a 

    valid threading context. System.Windows.Freezable inheritors should call this method at the 

    beginning of any API that writes to data members that are not dependency properties.
  """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __str__(self,*args):
  pass

class RenderMode(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies the rendering preference.

 

 enum RenderMode,values: Default (0),SoftwareOnly (1)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 Default=None
 SoftwareOnly=None
 value__=None





class AccessControlSections(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies which sections of a security descriptor to save or load.

 

 enum (flags) AccessControlSections,values: Access (2),All (15),Audit (1),Group (8),None (0),Owner (4)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 Access=None
 All=None
 Audit=None
 Group=None
 None=None
 Owner=None
 value__=None




class KeyEventArgs(EventArgs):
 """
 Provides data for the System.Windows.Forms.Control.KeyDown or System.Windows.Forms.Control.KeyUp event.

 

 KeyEventArgs(keyData: Keys)
 """
 @staticmethod
 def __new__(self,keyData):
  """ __new__(cls: type,keyData: Keys) """
  pass
 Alt=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value indicating whether the ALT key was pressed.



Get: Alt(self: KeyEventArgs) -> bool



"""

 Control=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value indicating whether the CTRL key was pressed.



Get: Control(self: KeyEventArgs) -> bool



"""

 Handled=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value indicating whether the event was handled.



Get: Handled(self: KeyEventArgs) -> bool



Set: Handled(self: KeyEventArgs)=value

"""

 KeyCode=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the keyboard code for a System.Windows.Forms.Control.KeyDown or System.Windows.Forms.Control.KeyUp event.



Get: KeyCode(self: KeyEventArgs) -> Keys



"""

 KeyData=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the key data for a System.Windows.Forms.Control.KeyDown or System.Windows.Forms.Control.KeyUp event.



Get: KeyData(self: KeyEventArgs) -> Keys



"""

 KeyValue=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the keyboard value for a System.Windows.Forms.Control.KeyDown or System.Windows.Forms.Control.KeyUp event.



Get: KeyValue(self: KeyEventArgs) -> int



"""

 Modifiers=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the modifier flags for a System.Windows.Forms.Control.KeyDown or System.Windows.Forms.Control.KeyUp event. The flags indicate which combination of CTRL,SHIFT,and ALT keys was pressed.



Get: Modifiers(self: KeyEventArgs) -> Keys



"""

 Shift=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value indicating whether the SHIFT key was pressed.



Get: Shift(self: KeyEventArgs) -> bool



"""

 SuppressKeyPress=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value indicating whether the key event should be passed on to the underlying control.



Get: SuppressKeyPress(self: KeyEventArgs) -> bool



Set: SuppressKeyPress(self: KeyEventArgs)=value

"""


